<?php

namespace App\Http\Controllers;

use App\Models\FeedbackQuestion;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\Admin\FeedbackQuestionRequest;
use Illuminate\Support\Facades\Schema;
use App\Http\Controllers\CompanyRoleController;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Artisan;



class FeedbackQuestionController extends CompanyRoleController
{
    protected static $title = 'Feedback Options';

    // Question types
    protected static $question_types = [
        '1' => 'Positive',
        '2' => 'Negative'
    ]; 

    // Issue types
    protected static $issue_types = [
        '1' => 'Maintenance',
        '2' => 'Cleanliness',
        '3' => 'Service'
    ]; 

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $feedback_questions=array();
        if (Schema::connection('mysql2')->hasTable('feedback_questions')) {
            $feedback_questions = FeedbackQuestion::paginate(Config::get('constants.page_limit'));
        }
        return view('admin.feedback_questions.index',['feedback_questions'=>$feedback_questions,'question_types'=>static::$question_types,'issue_types'=>static::$issue_types,'title'=>static::$title]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.feedback_questions.create',['question_types'=>static::$question_types,'issue_types'=>static::$issue_types,'title'=>static::$title]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(FeedbackQuestionRequest $request)
    {
        $feedback_question = FeedbackQuestion::create([
            'question' => strip_tags($request->question),
            'question_type' => $request->question_type,
            'issue_type' => $request->issue_type,
            'created_by' => Auth::user()->id
        ]);
        if($feedback_question)
        {
            return redirect()->route('feedback_questions.index')->with('message', 'Question added successfully.');
        }

        return redirect()->back()
        ->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\FeedbackQuestion  $feedback_question
     * @return \Illuminate\Http\Response
     */
    public function show(FeedbackQuestionRequest $feedback_question)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\FeedbackQuestion  $feedback_question
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        if($id)
        {
            $feedback_question = FeedbackQuestion::find($id);

            if($feedback_question)
            {
                return view('admin.feedback_questions.edit', ['feedback_question' => $feedback_question,'title'=>static::$title,'is_edit'=>1]);
            }
        }

        abort(404);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\FeedbackQuestion  $feedback_question
     * @return \Illuminate\Http\Response
     */
    public function update(FeedbackQuestionRequest $request, $id)
    {
        if($id)
        {
            $post = $request->all();
            $post['updated_by'] = Auth::user()->id;
            $feedback_question = FeedbackQuestion::find($id);

            if($post && $feedback_question)
            {
                if($feedback_question->update($post))
                {
                    return redirect()->route('feedback_questions.index')->with('message', 'Question edited successfully.');
                }
            }

            return redirect()->back()->with('error', __('messages.somethingWrong'));
        }

        abort(404);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\FeedbackQuestion  $feedback_question
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $up_data['deleted_by'] = Auth::user()->id;
        FeedbackQuestion::find($id)->update($up_data);

        FeedbackQuestion::find($id)->delete();
  
        return redirect()->back()->with('message', 'Question deleted successfully.');
    
    }
}
