<?php

namespace App\Http\Controllers;

use App\Models\Servey;
use App\Models\Questions;
use App\Models\QuestionOptions;
use App\Models\TemplateFeedbackQuestions;
use App\Models\SubQuestions;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\Admin\ServeyTemplateRequest;
use Illuminate\Support\Facades\Schema;
use App\Http\Controllers\CompanyRoleController;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Artisan;
use App\Models\FeedbackQuestion;
use Cviebrock\EloquentSluggable\Services\SlugService;



class ServeyTemplateController extends CompanyRoleController
{
    protected static $title = 'Questions';
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
    */

    // Question types
    protected static $question_types = [
        '1' => 'Smiley',
        '2' => 'Rating Bar',
        '3' => 'Multi Selection',
        '4' => 'Scale',
        '5' => 'Multi Options',
        '6' => 'Yes/No Question',
        '7' => 'Free Text'
    ]; 

    // Question tone
    protected static $question_tones = [
        '1' => 'Positive',
        '2' => 'Negative'
    ]; 

    public function view($servey_id)
    {
        $servey='';
        if (Schema::connection('mysql2')->hasTable('servey')) {
            $servey = Servey::where('id',$servey_id)->first();
        }
        $questions=array();
        if (Schema::connection('mysql2')->hasTable('questions')) {
            $questions = Questions::select('id','question','question_type','icon_type','order')->where('servey_id',$servey_id)->orderBy('order','ASC')->get()->toArray();
        }
        //return view('admin.servey.templates.index')->with(compact('servey','questions'));
        return view('admin.servey.templates.index',['servey'=>$servey,'questions'=>$questions,'question_types'=>static::$question_types,'title'=>static::$title]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create($servey_id)
    {
        $servey='';$feedback_questions=$positive_feedback_questions=$negative_feedback_questions=array();
        if (Schema::connection('mysql2')->hasTable('servey')) {
            $servey = Servey::where('id',$servey_id)->first();
            
            ///==== Start to Get Feedback Questions ====///////
            if (Schema::connection('mysql2')->hasTable('feedback_questions')) {
                $feedback_questions = FeedbackQuestion::where('active',1)->select('question','id','question_type')->get()->toArray();
                if(!empty($feedback_questions))
                {
                    foreach($feedback_questions as $f)
                    {
                        // ==== Get Postive Questions ====///
                        if($f['question_type']==1){
                            $positive_feedback_questions[$f['id']]=$f['question'];
                        }

                        // ==== Get Negative Questions ====///
                        if($f['question_type']==2){
                            $negative_feedback_questions[$f['id']]=$f['question'];
                        }
                    }
                }
                ///==== End to Get Feedback Questions ====///////
            }
        }
        return view('admin.servey.templates.question_type',['servey'=>$servey,'positive_feedback_questions'=>$positive_feedback_questions,'negative_feedback_questions'=>$negative_feedback_questions,'question_types'=>static::$question_types,'title'=>static::$title]);
    }

    function add_question($question_type,$servey_id)
    {
        if($question_type==1)
        {
            $title='Smiley';
        }else if($question_type==2){
            $title='Rating Bar';
        }else if($question_type==3){
            $title='Multi Selection';
        }else if($question_type==4){
            $title='Scale';
        }else if($question_type==5){
            $title='Multi Options';
        }else if($question_type==6){
            $title='Yes/No';
        }else if($question_type==7){
            $title='Free Text';
        }else if($question_type==8){
            $title='Single Option';
        }else if($question_type==8){
            $title='Single Option';
        }else if($question_type==9){
            $title='Multi Questions';
        }
        $servey='';$feedback_questions=$positive_feedback_questions=$negative_feedback_questions=array();
        if (Schema::connection('mysql2')->hasTable('servey')) {
            $servey = Servey::where('id',$servey_id)->first();
            
            if($question_type==1 || $question_type==2 || $question_type==4)
            {
                ///==== Start to Get Feedback Questions ====///////
                if (Schema::connection('mysql2')->hasTable('feedback_questions')) {
                    $feedback_questions = FeedbackQuestion::where('active',1)->select('question','id','question_type')->get()->toArray();
                    if(!empty($feedback_questions))
                    {
                        foreach($feedback_questions as $f)
                        {
                            // ==== Get Postive Questions ====///
                            if($f['question_type']==1){
                                $positive_feedback_questions[$f['id']]=$f['question'];
                            }

                            // ==== Get Negative Questions ====///
                            if($f['question_type']==2){
                                $negative_feedback_questions[$f['id']]=$f['question'];
                            }
                        }
                    }
                    ///==== End to Get Feedback Questions ====///////
                }
            }
        }
        return view('admin.servey.templates.create',['servey'=>$servey,'positive_feedback_questions'=>$positive_feedback_questions,'negative_feedback_questions'=>$negative_feedback_questions,'question_type'=>$question_type,'question_types'=>static::$question_types,'title'=>$title,'question_tones'=>static::$question_tones]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function save(ServeyTemplateRequest $request,$question_type,$servey_id)
    {
        //echo '<pre>';print_r($request->all());echo '</pre>';exit;
        //echo '<pre>';print_r($_POST);echo '</pre>';exit;
        $last_order = Questions::select('order')->where('servey_id',$servey_id)->orderBy('order', 'DESC')->first();
        if(!empty($last_order)){
            $order=$last_order->order + 1;
        }else{
            $order=1;
        }
        $question = Questions::create([
            'servey_id' => $servey_id,
            'question' => $request->question,
            //'description' => $request->description,
            'positive_title' => isset($request->positive_title) ? $request->positive_title : '',
            'negative_title' => isset($request->negative_title) ? $request->negative_title : '',
            'question_type' => $question_type,
            'is_multi_seleted' => ($question_type==3 || $question_type==5) ? 1 : 0,
            'other' => isset($request->other) ? 1 : 0,
            'order' => $order,
            'icon_type' => isset($request->image_type) ? $request->image_type : 0,
            'created_by' => Auth::user()->id,
        ]);
        if($question)
        {
            ////// Save multiple Options //////
            // ==== Allow only Question Type = Multi Options ======/
            if($question_type==5 || $question_type==8)
            {
                if(isset($request->items) && !empty($request->items))
                {
                    $option_data = [];
                    $k=1;
                    foreach ($request->items as $key => $items)
                    {
                        if(isset($items['option']))
                        {
                            $option_data[] = [
                                'question_id' => $question->id,
                                'option_id' => $k,
                                'options' => $items['option'],
                                'response_tone' => !empty($items['response_tone']) ? $items['response_tone'] : 1,
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }

                        $k++;
                    }
                    if(!empty($option_data) && count($option_data) > 0)
                    {
                        QuestionOptions::insert($option_data);
                    }
                }
                // $que_options_datas=array();
                // for($fq=1;$fq<=10;$fq++)
                // {
                //     $option_val = 'option'.$fq;
                //     $status_val = 'response_tone_name'.$fq;
                //     $option = $request->$option_val;
                //     if(isset($option) && !empty($option))
                //     {
                //         $status =$request->$status_val;
                //         $que_options_datas[] = [
                //             'question_id' => $question->id,
                //             'option_id' => $fq,
                //             'options' => $option,
                //             'response_tone' => !empty($status) ? $status : 1,
                //             'created_by' => Auth::user()->id,
                //             'created_at' => date('Y-m-d H:i:s'),
                //             'updated_at' => date('Y-m-d H:i:s')
                //         ];
                //     }
                // }
                // if(!empty($que_options_datas)) QuestionOptions::insert($que_options_datas);
            }

            ///// Save Positive/Negative Questions /////
            // ==== Allow only Question Type = Smiley & Rating Bar ======/
            if($question_type==1 || $question_type==2 || $question_type==4)
            {
                ///// Add Positive Feedback Questions //////
                if(isset($request->postive_questions) && !empty($request->postive_questions))
                {
                    $positive_questions=array();
                    foreach($request->postive_questions as $p)
                    {
                        if(!empty($p))
                        {
                            $positive_questions[] = [
                                'servey_id' => $servey_id,
                                'question_id' => $question->id,
                                'feedback_question_id' => $p,
                                'response_tone' => 1,
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                    }
                    if(!empty($positive_questions)) TemplateFeedbackQuestions::insert($positive_questions);
                }

                ////// save other option enable or not //////
                if(isset($request->other_positive) && !empty($request->other_positive)){
                    $other_positive = TemplateFeedbackQuestions::create([
                        'servey_id' => $servey_id,
                        'question_id' => $question->id,
                        'feedback_question_id' => 0,
                        'response_tone' => 1,
                        'created_by' => Auth::user()->id,
                        'created_at' => date('Y-m-d H:i:s'),
                        'updated_at' => date('Y-m-d H:i:s')
                    ]);
                }
                if(isset($request->other_negative) && !empty($request->other_negative)){
                    $other_positive = TemplateFeedbackQuestions::create([
                        'servey_id' => $servey_id,
                        'question_id' => $question->id,
                        'feedback_question_id' => 0,
                        'response_tone' => 2,
                        'created_by' => Auth::user()->id,
                        'created_at' => date('Y-m-d H:i:s'),
                        'updated_at' => date('Y-m-d H:i:s')
                    ]);
                }

                ///// Add Negative Feedback Questions //////
                if(isset($request->negative_questions) && !empty($request->negative_questions))
                {
                    $negative_questions=array();
                    foreach($request->negative_questions as $n)
                    {
                        if(!empty($n))
                        {
                            $negative_questions[] = [
                                'servey_id' => $servey_id,
                                'question_id' => $question->id,
                                'feedback_question_id' => $n,
                                'response_tone' => 2,
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                    }
                    if(!empty($negative_questions)) TemplateFeedbackQuestions::insert($negative_questions);
                }
            }
            
            ///// Save Multiple images with +/- /////
            // ==== Allow only Question Type = Multi Selection ======/
            if($question_type==3)
            {
                $option_img_data=array();
                $ii=1;
                for($img=1;$img<=4;$img++)
                {
                    if($request->file("option$img")){
                        
                        $status_val = 'response_tone_name'.$img;
                        $status =$request->$status_val;

                        $image_path = 'images/templates';
                        $image = UploadFile($request->file("option$img"),$image_path);

                        $option_img_data[] = [
                            'question_id' => $question->id,
                            'option_id' => $ii,
                            'options' => $image,
                            'response_tone' => !empty($status) ? $status : 1,
                            'created_by' => Auth::user()->id,
                            'created_at' => date('Y-m-d H:i:s'),
                            'updated_at' => date('Y-m-d H:i:s')
                        ];

                        $ii++;
                    }

                    
                }

                //// Insert Image options /////
                if(!empty($option_img_data)) QuestionOptions::insert($option_img_data);
            }

            //// Save Yes/No Question //////
            if($question_type==6)
            {
                ///// Save option for Yes /////   
                $yes_question = QuestionOptions::create([
                    'question_id' => $question->id,
                    'option_id' => 1,
                    'options' => '',
                    'response_tone' => !empty($request->yes_response_tone) ? $request->yes_response_tone : 1,
                    'created_by' => Auth::user()->id,
                    'created_at' => date('Y-m-d H:i:s'),
                    'updated_at' => date('Y-m-d H:i:s')
                ]);

                ///// Save option for no /////   
                $no_question = QuestionOptions::create([
                    'question_id' => $question->id,
                    'option_id' => 2,
                    'options' => '',
                    'response_tone' => !empty($request->no_response_tone) ? $request->no_response_tone : 2,
                    'created_by' => Auth::user()->id,
                    'created_at' => date('Y-m-d H:i:s'),
                    'updated_at' => date('Y-m-d H:i:s')
                ]); 
            }

            if($question_type==9)
            {
                //echo '<pre>';print_r($request->items);echo '</pre>';exit;
                if(isset($request->items) && !empty($request->items))
                {
                    $option_data = [];
                    $k=1;
                    foreach ($request->items as $key => $items)
                    {
                        if(isset($items['option']))
                        {
                            $option_data[] = [
                                'question_id' => $question->id,
                                'option_id' => $k+1,
                                'options' => $items['option'],
                                'response_tone' => !empty($items['response_tone']) ? $items['response_tone'] : 1,
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                    }
                    if(!empty($option_data) && count($option_data) > 0)
                    {
                        QuestionOptions::insert($option_data);
                    }
                }

                if(isset($request->questions) && !empty($request->questions))
                {
                    $question_data = [];
                    foreach ($request->questions as $key => $sq)
                    {
                        if(isset($sq['sub_questions']))
                        {
                            $question_data[] = [
                                'question_id' => $question->id,
                                'servey_id' => $servey_id,
                                'subquestion' => $sq['sub_questions'],
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                    }
                    if(!empty($question_data) && count($question_data) > 0)
                    {
                        SubQuestions::insert($question_data);
                    }
                }
            }
            return redirect()->route('servey.template',[$servey_id])->with('message', 'Question added successfully.');
        }

        return redirect()->back()
        ->withInput();
    }

    /* Save Free text question */
    function save_free_text($question_type,$servey_id)
    {
        $question = Questions::create([
            'servey_id' => $servey_id,
            'question' => '',
            'positive_title' => '',
            'negative_title' => '',
            'question_type' => $question_type,
            'is_multi_seleted' =>  0,
            'icon_type' => 0,
            'created_by' => Auth::user()->id,
        ]);
        if($question)
        {
            return redirect()->route('servey.template',[$servey_id])->with('message', 'Question added successfully.');
        }
        return redirect()->back()
        ->withInput();
    }
    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    // public function show(Servey $servey)
    // {
    //     echo 'f';exit;
    // }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        if($id)
        {
            $servey = Servey::find($id);

            if($servey)
            {
                return view('admin.servey.edit', ['servey' => $servey,'title'=>static::$title,'is_edit'=>1]);
            }
        }

        abort(404);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function update(ServeyTemplateRequest $request,$question_type, $id,$servey_id)
    {
        if($id)
        {
            $post = $request->all();
            //echo '<pre>';print_r($post);echo '</pre>';exit;
            $post['other'] = isset($request->other) ? 1 : 0;
            $post['icon_type'] = isset($request->image_type) ? $request->image_type : 0;
            $post['updated_by'] = Auth::user()->id;
            $question = Questions::find($id);

            if($post && $question)
            {
                if($question->update($post))
                {
                    if($question_type==5 || $question_type==8)
                    {
                        if(isset($request->items) && !empty($request->items))
                        {
                            $option_data = $keep_items = [];
                            $k=1;
                            foreach ($request->items as $key => $items)
                            {
                                $items['option_id'] = (isset($items['option_id']) && !empty($items['option_id'])) ? $items['option_id'] : 0;
                                $poItem = QuestionOptions::updateOrCreate([
                                    'id' => $items['id']
                                ], [
                                    'question_id' => $question->id,
                                    'option_id' => $k+1,
                                    'options' => $items['option'],
                                    'response_tone' => !empty($items['response_tone']) ? $items['response_tone'] : 1,
                                    'updated_by' => Auth::user()->id,
                                    'updated_at' => date('Y-m-d H:i:s')
                                ]);

                                $keep_items[] = $poItem->id;

                                $k++;
                            }
                            if(isset($keep_items) && !empty($keep_items))
                            {
                                QuestionOptions::where('question_id', $question->id)->whereNotIn('id', $keep_items)->delete();
                            }
                        }
                        // $que_options_datas=array();
                        // for($fq=1;$fq<=10;$fq++)
                        // {
                        //     $option_val = 'option'.$fq;
                        //     $status_val = 'response_tone_name'.$fq;
                        //     $option = $request->$option_val;
                        //     if(isset($option) && !empty($option))
                        //     {
                        //         $status =$request->$status_val;
                        //         $que_options = QuestionOptions::updateOrCreate(
                        //         [
                        //             'question_id'   => $question->id,
                        //             'option_id' => $fq
                        //         ],
                        //         [
                        //             'question_id' => $question->id,
                        //             'option_id' => $fq,
                        //             'options' => $option,
                        //             'response_tone' => !empty($status) ? $status : 1,
                        //             'created_by' => Auth::user()->id,
                        //             'created_at' => date('Y-m-d H:i:s'),
                        //             'updated_at' => date('Y-m-d H:i:s')
                        //         ],
                        //         );
                        //     }
                        // }
                        //if(!empty($que_options_datas)) QuestionOptions::insert($que_options_datas);
                    }

                    ///// Save Positive/Negative Questions /////
                    // ==== Allow only Question Type = Smiley & Rating Bar ======/
                    if($question_type==1 || $question_type==2 || $question_type==4)
                    {
                        $feedback_questions = TemplateFeedbackQuestions::select('feedback_question_id')->where('question_id',$question->id)->get()->pluck('feedback_question_id','feedback_question_id');
                        $feedback_question_index = !empty($feedback_questions) ? array_values($feedback_questions->toArray()) : [];
                        ///// Add Positive Feedback Questions //////
                        $pids=array();
                        if(isset($request->postive_questions) && !empty($request->postive_questions))
                        {
                            $positive_questions=array();
                            foreach($request->postive_questions as $p)
                            {
                                if(!empty($p))
                                {
                                    $que_options = TemplateFeedbackQuestions::updateOrCreate(
                                    [
                                        'question_id'   => $question->id,
                                        'feedback_question_id' => $p
                                    ],
                                    [
                                        'servey_id' => $servey_id,
                                        'question_id' => $question->id,
                                        'feedback_question_id' => $p,
                                        'response_tone' => 1,
                                        'updated_by' => Auth::user()->id,
                                        'updated_at' => date('Y-m-d H:i:s'),
                                        'deleted_at' => NULL
                                    ],
                                    ); 
                                    $pids[] = $p;
                                }
                            }
                        }
                        
                        ////// save other option enable or not //////
                        if(isset($request->other_positive) && !empty($request->other_positive)){
                            $other_positive = TemplateFeedbackQuestions::updateOrCreate(
                                [
                                    'question_id'   => $question->id,
                                    'feedback_question_id' => 0,
                                    'response_tone' => 1,
                                ],
                                [
                                    'servey_id' => $servey_id,
                                    'question_id' => $question->id,
                                    'feedback_question_id' => 0,
                                    'response_tone' => 1,
                                    'created_by' => Auth::user()->id,
                                    'created_at' => date('Y-m-d H:i:s'),
                                    'updated_at' => date('Y-m-d H:i:s')
                                ]
                                );
                        }else{
                            TemplateFeedbackQuestions::where('question_id',$question->id)->where('feedback_question_id',0)->where('response_tone',1)->delete(); 
                        }
                        if(isset($request->other_negative) && !empty($request->other_negative)){
                            $other_positive = TemplateFeedbackQuestions::updateOrCreate(
                                [
                                    'question_id'   => $question->id,
                                    'feedback_question_id' => 0,
                                    'response_tone' => 2,
                                ],
                                [
                                'servey_id' => $servey_id,
                                'question_id' => $question->id,
                                'feedback_question_id' => 0,
                                'response_tone' => 2,
                                'created_by' => Auth::user()->id,
                                'updated_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s'),
                                'deleted_at' => NULL
                                ]);
                        }else{
                            TemplateFeedbackQuestions::where('question_id',$question->id)->where('feedback_question_id',0)->where('response_tone',2)->delete(); 
                        }

                        ///// Add Negative Feedback Questions //////
                        if(isset($request->negative_questions) && !empty($request->negative_questions))
                        {
                            $negative_questions=array();
                            foreach($request->negative_questions as $n)
                            {
                                if(!empty($n))
                                {
                                    $que_options = TemplateFeedbackQuestions::updateOrCreate(
                                        [
                                            'question_id'   => $question->id,
                                            'feedback_question_id' => $n
                                        ],
                                        [
                                            'servey_id' => $servey_id,
                                            'question_id' => $question->id,
                                            'feedback_question_id' => $n,
                                            'response_tone' => 2,
                                            'updated_by' => Auth::user()->id,
                                            'created_by' => Auth::user()->id,
                                            'created_at' => date('Y-m-d H:i:s'),
                                            'updated_at' => date('Y-m-d H:i:s')
                                        ],
                                    );
                                }
                                $pids[] = $n;
                            }
                        }

                        if(!empty($feedback_question_index) && !empty($pids))
                        {
                            $feedback_question_index = array_filter($feedback_question_index, function($a) { return ($a !== 0); });
                            // echo '<pre>';print_r($feedback_question_index);
                            // echo '<pre>';print_r($pids);exit;
                            $delete_ids = array_diff($feedback_question_index,$pids);
                            if(!empty($delete_ids))
                            {
                                TemplateFeedbackQuestions::where('question_id',$question->id)->whereIn('feedback_question_id',$delete_ids)->delete();
                            }
                        }
                        //echo '<pre>';print_r($result);exit;
                    }

                    //// Save Yes/No Question //////
                    if($question_type==6)
                    {
                        ///// Save option for Yes /////   
                        $yes_question = QuestionOptions::updateOrCreate(
                            [
                                'question_id' => $question->id,
                                'option_id' => 1,
                            ],
                            [
                            'question_id' => $question->id,
                            'option_id' => 1,
                            'options' => '',
                            'response_tone' => !empty($request->yes_response_tone) ? $request->yes_response_tone : 1,
                            'created_by' => Auth::user()->id,
                            'created_at' => date('Y-m-d H:i:s'),
                            'updated_at' => date('Y-m-d H:i:s')
                        ]);

                        ///// Save option for no /////   
                        $no_question = QuestionOptions::updateOrCreate(
                            [
                                'question_id' => $question->id,
                                'option_id' => 2,
                            ],
                            [
                            'question_id' => $question->id,
                            'option_id' => 2,
                            'options' => '',
                            'response_tone' => !empty($request->no_response_tone) ? $request->no_response_tone : 2,
                            'created_by' => Auth::user()->id,
                            'created_at' => date('Y-m-d H:i:s'),
                            'updated_at' => date('Y-m-d H:i:s')
                            ]); 
                    }

                    ///// Save Multiple images with +/- /////
                    // ==== Allow only Question Type = Multi Selection ======/
                    if($question_type==3)
                    {
                        $option_img_data=array();
                        for($img=1;$img<=4;$img++)
                        {
                            $check_img_exist = QuestionOptions::where('question_id',$question->id)->orderBy('option_id','desc')->first();

                            $img_exist = QuestionOptions::where('question_id',$question->id)->where('option_id',$img)->first();
                            
                            $status_val = 'response_tone_name'.$img;
                            $status =$request->$status_val;

                            if($request->file("option$img")){
                                
                                $image_path = 'images/templates';
                                $image = UploadFile($request->file("option$img"),$image_path);
                            }else{$image='';}

                            if(!empty($img_exist)){
                                //// Update /////
                                $up_data = [
                                    'question_id' => $question->id,
                                    'response_tone' => !empty($status) ? $status : 1
                                ];
                                if(!empty($image)){
                                    $up_data['options'] = $image;
                                }
                                QuestionOptions::where('question_id',$question->id)->where('option_id',$img)->update($up_data);
                            }else{
                                //// Create /////
                                if($request->file("option$img")){
                                    $add_data = [
                                        'question_id' => $question->id,
                                        'option_id' => $img,
                                        'response_tone' => !empty($status) ? $status : 1
                                    ];
                                    if(!empty($image)){
                                        $add_data['options'] = $image;
                                    }
                                    QuestionOptions::insert($add_data);
                                }
                            }
                            
                            

                            // if($request->file("option$img")){
                                
                            //     $image_path = 'images/templates';
                            //     $image = UploadFile($request->file("option$img"),$image_path);

                            //     $option_img_data = QuestionOptions::updateOrCreate(
                            //         [
                            //             'question_id' => $question->id,
                            //             'option_id' => $img,
                            //         ],
                            //         [
                            //         'question_id' => $question->id,
                            //         'option_id' => $index,
                            //         'options' => $image,
                            //         'response_tone' => !empty($status) ? $status : 1
                            //     ]);
                            // }

                            
                            
                            // if(!empty($status) && !empty($check_img_exist)){
                                
                            //     $option_status_data = QuestionOptions::updateOrCreate(
                            //         [
                            //             'question_id' => $question->id,
                            //             'option_id' => $img,
                            //         ],
                            //         [
                            //         'response_tone' => !empty($status) ? $status : 1
                            //     ]);
                            // }

                        }

                    }

                    if($question_type==9)
                    {
                        if(isset($request->items) && !empty($request->items))
                        {
                            $option_data = $keep_items = [];
                            $k=1;
                            foreach ($request->items as $key => $items)
                            {
                                // if(isset($items['option']))
                                // {
                                //     $option_data[] = [
                                //         'question_id' => $question->id,
                                //         'option_id' => $k+1,
                                //         'options' => $items['option'],
                                //         'response_tone' => !empty($items['response_tone']) ? $items['response_tone'] : 1,
                                //         'updated_by' => Auth::user()->id,
                                //         'updated_at' => date('Y-m-d H:i:s')
                                //     ];
                                // }

                                //echo '<pre>';print_r($option_data);echo '</pre>';
                                $items['option_id'] = (isset($items['option_id']) && !empty($items['option_id'])) ? $items['option_id'] : 0;
                                $poItem = QuestionOptions::updateOrCreate([
                                    'id' => $items['id']
                                ], [
                                    'question_id' => $question->id,
                                    'option_id' => $k+1,
                                    'options' => $items['option'],
                                    'response_tone' => !empty($items['response_tone']) ? $items['response_tone'] : 1,
                                    'updated_by' => Auth::user()->id,
                                    'updated_at' => date('Y-m-d H:i:s')
                                ]);

                                $keep_items[] = $poItem->id;

                                $k++;
                            }
                            if(isset($keep_items) && !empty($keep_items))
                            {
                                QuestionOptions::where('question_id', $question->id)->whereNotIn('id', $keep_items)->delete();
                            }
                        }

                        if(isset($request->questions) && !empty($request->questions))
                        {
                            $keep_qitems = [];
                            foreach ($request->questions as $key => $q)
                            {
                                if(isset($q['sub_questions']) && !empty($q['sub_questions']))
                                {
                                    //echo '<pre>';print_r($option_data);echo '</pre>';
                                    $Que_da = SubQuestions::updateOrCreate([
                                        'id' => $q['id']
                                    ], [
                                        'servey_id' => $servey_id,
                                        'question_id' => $question->id,
                                        'subquestion' => $q['sub_questions'],
                                        'updated_by' => Auth::user()->id,
                                        'updated_at' => date('Y-m-d H:i:s')
                                    ]);

                                    $keep_qitems[] = $Que_da->id;

                                }
                            }
                            if(isset($keep_qitems) && !empty($keep_qitems))
                            {
                                SubQuestions::where('question_id', $question->id)->whereNotIn('id', $keep_qitems)->delete();
                            }
                        }
                    }

                    return redirect()->route('servey.template.details',$id)->with('message', 'Question edited successfully.');
                }
            }

            return redirect()->back()->with('error', __('messages.somethingWrong'));
        }

        abort(404);
    }


    function update_order(Request $request,$servey_id)
    {
        if (Schema::connection('mysql2')->hasTable('questions')) {
            //$posts = Questions::all();
            $posts = Questions::select('order','id')->where('servey_id',$servey_id)->get();

            $data=$final=array();
            foreach ($posts as $post) {
                foreach ($request->order as $order) {
    
                    if ($order['id'] == $post->id) {
                        // if($order['position'] == 1)
                        // {
                        //     $question = Questions::select('id','order','question','question_type')->where('id',$post->id)->first();
                        //     if($question->question_type==3 || $question->question_type==5)
                        //     {
                        //         return response("Can't swipe.", 200);
                        //     }
                        // }
                        // $data[$order['position']] = $post->id;
                        // $final[] = $data;
                        $post->update(['order' => $order['position']]);
                    }
                }
            }
            //print_r($questions);exit;
        }
        return response('Update Successfully.', 200);
    }
    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $up_data['deleted_by'] = Auth::user()->id;
        Servey::find($id)->update($up_data);

        Servey::find($id)->delete();
  
        return redirect()->back()->with('message', 'Servey deleted successfully.');
    
    }

    public function delete_question($id)
    {
        Questions::find($id)->delete();
  
        return redirect()->back()->with('message', 'Question deleted successfully.');
    }
    public function delete_option($id)
    {
        QuestionOptions::find($id)->delete();
  
        return redirect()->back()->with('message', 'Option deleted successfully.');
    }
    

    /* Duplicate sevey */
    public function duplicate($id)
    {
        if($id)
        {
            $servey = Servey::find($id);
            if($servey)
            {
                $slug = SlugService::createSlug(Servey::class, 'slug', $servey->name);
                $d_servey = Servey::create([
                    'name' => $servey->name,
                    'message' => $servey->message,
                    'slug' => $slug,
                    'created_by' => Auth::user()->id
                ]);

                if($d_servey)
                {
                    ///// Get servey Questions //////
                    $questions = Questions::where('servey_id',$id)->get();
                    $que_data=$question_option_data=$feedback_que_data='';
                    foreach($questions as $question)
                    {
                        $que_data = $question->replicate();
                        $que_data->created_by = Auth::user()->id;
                        $que_data->servey_id = $d_servey->id;
                        $que_data->created_at = now();
                        $duplicate_question = $que_data->save();
                        
                        if($duplicate_question)
                        {
                            $question_options='';
                            $question_option_data='';
                            ////// Insert Question Options /////////
                            $question_options = QuestionOptions::where('question_id',$question->id)->get();
                            if(!empty($question_options))
                            {
                                foreach($question_options as $opt)
                                {
                                    $question_option_data = $opt->replicate();
                                    $question_option_data->question_id = $que_data->id;
                                    $question_option_data->created_at = now();
                                    $question_option_data->save();
                                }
                            }
                            $feedback_que_data='';
                            /////// Insert +/- questions if any //////
                            $feedback_que = TemplateFeedbackQuestions::where('question_id',$question->id)->get();
                            if(!empty($feedback_que))
                            {
                                foreach($feedback_que as $f)
                                {
                                    $feedback_que_data = $f->replicate();
                                    $feedback_que_data->question_id = $que_data->id;
                                    $feedback_que_data->servey_id = $d_servey->id;
                                    $feedback_que_data->created_at = now();
                                    $feedback_que_data->save();
                                }
                            }
                        }
                    }

                    //// Save cofiguarations //////
                    $configurations = \App\Models\ServeyConfig::where('servey_id',$servey->id)->get();
                    if(!empty($configurations))
                    {
                        foreach($configurations as $cg)
                        {
                            $config_data = $cg->replicate();
                            $config_data->servey_id=$d_servey->id;
                            $config_data->created_by=Auth::user()->id;
                            $config_data->created_at=$config_data->updated_at=date('Y-m-d H:i:s');
                            $save_config = $config_data->save();
                        }
                    }
                    
                    //// Save advertisement //////
                    // $advertise =\App\Models\MonitorAdvert::where('servey_id',$id)->get();
                    // if($advertise)
                    // {
                    //     foreach($advertise as $ad)
                    //     {
                    //         $advertise_data = $ad->replicate();
                    //         $advertise_data->monitor_id=$d_servey->id;
                    //         $advertise_data->created_by=Auth::user()->id;
                    //         $advertise_data->created_at=$config_data->updated_at=date('Y-m-d H:i:s');
                    //         $save_advert = $advertise_data->save();
                    //     } 
                    // }
                    return redirect()->route('servey.index')->with('message', 'Servey duplicated successfully.');
                }
            }
        }
        abort(404);
    }

    /* Question Details */
    function details($id)
    {
        $details = Questions::with('options','servey_feedback_questions')->where('id',$id)->get()->toArray();
        $sub_questions = SubQuestions::where('question_id',$id)->get()->toArray();
        //echo '<pre>';print_r($sub_questions);echo '</pre>';exit;
        $feedback_ids=array();
        if(!empty($details[0]['servey_feedback_questions']))
        {
            foreach($details[0]['servey_feedback_questions'] as $fds)
            {
                $feedback_ids[] = $fds['feedback_question_id'];
            }
        }
        if($details[0]['question_type']==1)
        {
            $title='Smiley';
        }else if($details[0]['question_type']==2){
            $title='Rating Bar';
        }else if($details[0]['question_type']==3){
            $title='Multi Selection';
        }else if($details[0]['question_type']==4){
            $title='Scale';
        }else if($details[0]['question_type']==5){
            $title='Multi Options';
        }else if($details[0]['question_type']==6){
            $title='Yes/No';
        }else if($details[0]['question_type']==7){
            $title='Free Text';
        }else if($details[0]['question_type']==8){
            $title='Single Options';
        }else if($details[0]['question_type']==9){
            $title='Multi Questions';
        }

        $pos_search = ['feedback_question_id' => 0, 'response_tone' => 1];
        $other_positive = array_keys(
            array_filter(
                $details[0]['servey_feedback_questions'],
                function ($v) use ($pos_search) { return $v['feedback_question_id'] == $pos_search['feedback_question_id'] && $v['response_tone'] == $pos_search['response_tone']; }
            )
        );

        $neg_search = ['feedback_question_id' => 0, 'response_tone' => 2];
        $other_negative = array_keys(
            array_filter(
                $details[0]['servey_feedback_questions'],
                function ($v) use ($neg_search) { return $v['feedback_question_id'] == $neg_search['feedback_question_id'] && $v['response_tone'] == $neg_search['response_tone']; }
            )
        );
        $servey='';$feedback_questions=$positive_feedback_questions=$negative_feedback_questions=array();
        if($details[0]['question_type']==1 || $details[0]['question_type']==2 || $details[0]['question_type']==4)
        {
            ///==== Start to Get Feedback Questions ====///////
            if (Schema::connection('mysql2')->hasTable('feedback_questions')) {
                $feedback_questions = FeedbackQuestion::where('active',1)->select('question','id','question_type')->get()->toArray();
                if(!empty($feedback_questions))
                {
                    foreach($feedback_questions as $f)
                    {
                        // ==== Get Postive Questions ====///
                        if($f['question_type']==1){
                            $positive_feedback_questions[$f['id']]=$f['question'];
                        }

                        // ==== Get Negative Questions ====///
                        if($f['question_type']==2){
                            $negative_feedback_questions[$f['id']]=$f['question'];
                        }
                    }
                }
                ///==== End to Get Feedback Questions ====///////
            }
        }
        //echo '<pre>';print_r($details[0]);echo '</pre>';
        return view('admin.servey.templates.details', ['details'=>$details,'sub_questions'=>$sub_questions,'feedback_ids'=>$feedback_ids,'title'=>$title,'positive_feedback_questions'=>$positive_feedback_questions,'negative_feedback_questions'=>$negative_feedback_questions,'question_tones'=>static::$question_tones,'other_positive'=> (!empty($other_positive) ? 1 : 0),'other_negative'=> (!empty($other_negative) ? 1 : 0)]);
    }
}
