<?php

namespace App\Http\Controllers;

use App\Models\Site;
use App\Models\SiteUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\Admin\SiteRequest;
use Cviebrock\EloquentSluggable\Services\SlugService;
use Illuminate\Support\Facades\Schema;
use App\Http\Controllers\CompanyRoleController;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Artisan;



class SiteController extends CompanyRoleController
{
    protected static $title = 'Sites';
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $sites=array();
        if (Schema::connection('mysql2')->hasTable('sites')) {
            //// if site manager ////
            if(Auth::user()->role==3)
            {
                $sites = Site::select('sites.*')->join('site_users','site_users.site_id','=','sites.id')->where('site_users.user_id',Auth::user()->id)->paginate(Config::get('constants.page_limit'));
            }else{
                $sites = Site::paginate(Config::get('constants.page_limit'));
            }
        }
        return view('admin.sites.index',['sites'=>$sites,'title'=>static::$title,'role'=>Auth::user()->role]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        
        // if (!Schema::connection('mysql2')->hasTable($table_name)) {
        //     Schema::create( 'site_users', function ( $table ) {
        //         $table->increments('id');
        //         $table->string( 'name' );
        //         $table->string( 'random' );
        //     });
        // }
        if(isAdminPermission() == true)
        {
            return view('admin.sites.create',['title'=>static::$title]);
        }else{
            abort(404);
        } 
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(SiteRequest $request)
    {
        // $validated = $request->validate([
        //     'name' => 'required|max:255|unique:sites,name',
        //     'location'=>'required'
        // ]);

        // $site = new Site();
        // $add_site = $site->create($request);
        // // return $add_site;
        $slug = SlugService::createSlug(Site::class, 'slug', $request->name);
        $site = Site::insert([
            'name' => $request->name,
            'slug' => $slug,
            'location' => $request->location,
            'company_id' => Auth::user()->id,
            'created_by' => Auth::user()->id,
            'created_at' => date('Y-m-d H:i:s'),
        ]);
        if($site)
        {
            return redirect()->route('sites.index')->with('message', 'Site added successfully.');
        }

        return redirect()->back()
        ->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Site  $site
     * @return \Illuminate\Http\Response
     */
    public function show(Site $site)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Site  $site
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        if($id)
        {
            $site = Site::find($id);

            if($site)
            {
                return view('admin.sites.edit', ['site' => $site,'title'=>static::$title,'is_edit'=>1]);
            }
        }

        abort(404);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Site  $site
     * @return \Illuminate\Http\Response
     */
    public function update(SiteRequest $request, $id)
    {
        if($id)
        {
            $post = $request->all();
            $post['updated_by'] = Auth::user()->id;

            $site = Site::find($id);

            if($post && $site)
            {
                if($site->update($post))
                {
                    return redirect()->route('sites.index')->with('message', 'Site edited successfully.');
                }
            }

            return redirect()->back()->with('error', __('messages.somethingWrong'));
        }

        abort(404);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Site  $site
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        /////// Only admin can delete the site ////
        if(Auth::user()->role==2)
        {
            $up_data['deleted_by'] = Auth::user()->id;
            Site::find($id)->update($up_data);

            $site = Site::find($id)->delete();

            if($site)
            {
                $up_data['deleted_by'] = Auth::user()->id;
                SiteUser::where('site_id',$id)->update($up_data);

                SiteUser::where('site_id',$id)->delete();
            }
        }else{
            abort(404);
        }
        
        return redirect()->back()->with('message', 'Site deleted successfully.');
    
    }
}
