<?php

namespace App\Http\Controllers\api;

use Validator;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Str; 
use App\api_helper;
use Config;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Schema\Blueprint;
use App\Models\User;

class SyncDB extends Controller
{
    function sync_data()
    {
        $users = User::select('id','db_name')->where('db_name','<>',NULL)->groupBy('db_name')->get()->toArray();
        //echo '<pre>';print_r($users);exit;
        if(!empty($users))
        {
            DB::disconnect('mysql');
            foreach($users as $u)
            {
                $db_name = $u['db_name'];
                //// connect to DB ///////
                Config::set('database.connections.mysql2.database', $db_name);
                
                //// add tables if not exist /////
                if (!Schema::connection('mysql2')->hasTable('sites')) {
                    Schema::connection('mysql2')->create('sites', function (Blueprint $table) {
                        $table->increments('id');
                        $table->bigInteger('company_id')->nullable();
                        $table->string('name')->nullable();
                        $table->string('location')->nullable();
                        $table->string('slug')->nullable();
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('site_users')) {
                    Schema::connection('mysql2')->create('site_users', function (Blueprint $table) {
                        $table->increments('id');
                        $table->bigInteger('site_id')->nullable();
                        $table->string('user_id')->nullable();
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('servey')) {
                    Schema::connection('mysql2')->create('servey', function (Blueprint $table) {
                        $table->increments('id');
                        $table->string('name')->nullable();
                        $table->integer('type')->nullable()->comment('1 : portrait, 2 : landscape');
                        $table->text('size')->nullable();
                        $table->text('message')->nullable()->comment('Display thankyou msg in app after giving rating.');
                        $table->string('slug')->nullable();
                        $table->integer('ask_customer_details')->default(0);
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('ratings')) {
                    Schema::connection('mysql2')->create('ratings', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('company_id')->nullable();
                        $table->integer('site_id')->nullable();
                        $table->integer('monitor_id')->nullable();
                        $table->integer('servey_id')->nullable();
                        $table->json('servey_response')->nullable();
                        $table->integer('positive')->default(0);
                        $table->integer('negative')->default(0);
                        $table->integer('nuetral')->default(0);
                        $table->decimal('positive_avg',5,2)->nullable();
                        $table->decimal('negative_avg',5,2)->nullable();
                        $table->decimal('neutral_avg',5,2)->nullable();
                        $table->decimal('positive_per',5,2)->nullable();
                        $table->decimal('negative_per',5,2)->nullable();
                        $table->decimal('neutral_per',5,2)->nullable();
                        $table->integer('question_count')->default(0);
                        $table->integer('performance')->nullable()->comment('1:Good,2:Fair,3:Poor');
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('hourly_ratings')) {
                    Schema::connection('mysql2')->create('hourly_ratings', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('company_id')->nullable();
                        $table->integer('site_id')->nullable();
                        $table->integer('monitor_id')->nullable();
                        $table->integer('rating')->nullable();
                        $table->date('date')->nullable();
                        $table->time('time')->nullable();
                        $table->integer('0_hr')->default(0);
                        $table->integer('1_hr')->default(0);
                        $table->integer('2_hr')->default(0);
                        $table->integer('3_hr')->default(0);
                        $table->integer('4_hr')->default(0);
                        $table->integer('5_hr')->default(0);
                        $table->integer('6_hr')->default(0);
                        $table->integer('7_hr')->default(0);
                        $table->integer('8_hr')->default(0);
                        $table->integer('9_hr')->default(0);
                        $table->integer('10_hr')->default(0);
                        $table->integer('11_hr')->default(0);
                        $table->integer('12_hr')->default(0);
                        $table->integer('13_hr')->default(0);
                        $table->integer('14_hr')->default(0);
                        $table->integer('15_hr')->default(0);
                        $table->integer('16_hr')->default(0);
                        $table->integer('17_hr')->default(0);
                        $table->integer('18_hr')->default(0);
                        $table->integer('19_hr')->default(0);
                        $table->integer('20_hr')->default(0);
                        $table->integer('21_hr')->default(0);
                        $table->integer('22_hr')->default(0);
                        $table->integer('23_hr')->default(0);
                        $table->timestamps();
                    });
                }  

                if (!Schema::connection('mysql2')->hasTable('hourly_question_ratings')) {
                    Schema::connection('mysql2')->create('hourly_question_ratings', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('site_id')->nullable();
                        $table->integer('monitor_id')->nullable();
                        $table->integer('servey_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->integer('rating')->nullable();
                        $table->date('date')->nullable();
                        $table->time('time')->nullable();
                        $table->integer('0_hr')->default(0);
                        $table->integer('1_hr')->default(0);
                        $table->integer('2_hr')->default(0);
                        $table->integer('3_hr')->default(0);
                        $table->integer('4_hr')->default(0);
                        $table->integer('5_hr')->default(0);
                        $table->integer('6_hr')->default(0);
                        $table->integer('7_hr')->default(0);
                        $table->integer('8_hr')->default(0);
                        $table->integer('9_hr')->default(0);
                        $table->integer('10_hr')->default(0);
                        $table->integer('11_hr')->default(0);
                        $table->integer('12_hr')->default(0);
                        $table->integer('13_hr')->default(0);
                        $table->integer('14_hr')->default(0);
                        $table->integer('15_hr')->default(0);
                        $table->integer('16_hr')->default(0);
                        $table->integer('17_hr')->default(0);
                        $table->integer('18_hr')->default(0);
                        $table->integer('19_hr')->default(0);
                        $table->integer('20_hr')->default(0);
                        $table->integer('21_hr')->default(0);
                        $table->integer('22_hr')->default(0);
                        $table->integer('23_hr')->default(0);
                        $table->timestamps();
                    });
                }               

                if (!Schema::connection('mysql2')->hasTable('rating_customer_details')) {
                    Schema::connection('mysql2')->create('rating_customer_details', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('rating_id')->nullable();
                        $table->string('name')->nullable();
                        $table->string('email')->nullable();
                        $table->string('phone_no')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('feedback_questions')) {
                    Schema::connection('mysql2')->create('feedback_questions', function (Blueprint $table) {
                        $table->increments('id');
                        $table->text('question')->nullable();
                        $table->string('question_type')->nullable()->comment('1:positive,2:negative');;
                        $table->string('issue_type')->nullable()->comment('1:maintenance,2:cleanliness,3:service');;
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->integer('active')->default(1);
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('questions')) {
                    Schema::connection('mysql2')->create('questions', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->string('question')->nullable();
                        $table->string('description')->nullable();
                        $table->string('positive_title')->nullable();
                        $table->string('negative_title')->nullable();
                        $table->string('icon_id')->nullable();
                        $table->string('question_type')->nullable()->comment('1=smiley,2=rating bar,3=multiple image selections,4=scale,5=multiple options ');
                        $table->integer('icon_type')->nullable()->comment('for smiley 1 = 1-2-3,for rating bar=1-2-3-4');
                        $table->integer('is_multi_seleted')->default(0);
                        $table->integer('other')->default(0);
                        $table->integer('active')->default(1);
                        $table->integer('order')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('question_options')) {
                    Schema::connection('mysql2')->create('question_options', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('question_id')->nullable();
                        $table->integer('option_id')->nullable();
                        $table->string('options')->nullable();
                        $table->string('response_tone')->nullable();
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('servey_feedback_questions')) {
                    Schema::connection('mysql2')->create('servey_feedback_questions', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->integer('feedback_question_id')->nullable();
                        $table->integer('response_tone')->nullable()->comment('1=positive,2=negative');
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('monitor_advertise')) {
                    Schema::connection('mysql2')->create('monitor_advertise', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('monitor_id')->nullable();
                        $table->text('image')->nullable();
                        $table->integer('sort')->nullable();
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('monitor_config')) {
                    Schema::connection('mysql2')->create('monitor_config', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('monitor_id')->nullable();
                        $table->integer('config_id')->nullable();
                        $table->enum('date_format',['1', '2'])->nullable()->comment('1 : dd/mm/yy, 2 : mm/dd/yy');
                        $table->enum('clock_format',['1', '2'])->nullable()->comment('1 : 12hr, 2 : 24hr');
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('servey_config')) {
                    Schema::connection('mysql2')->create('servey_config', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('config_id')->nullable();
                        $table->enum('date_format',['1', '2'])->nullable()->comment('1 : dd/mm/yy, 2 : mm/dd/yy');
                        $table->enum('clock_format',['1', '2'])->nullable()->comment('1 : 12hr, 2 : 24hr');
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('servey_advertise')) {
                    Schema::connection('mysql2')->create('servey_advertise', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->text('image')->nullable();
                        $table->integer('sort')->nullable();
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('rating_question_details')) {
                    Schema::connection('mysql2')->create('rating_question_details', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->integer('response_tone')->nullable();
                        $table->integer('rating')->nullable();
                        // $table->text('option_ids')->nullable();
                        // $table->text('options_value')->nullable();
                        // $table->text('feedback_question_ids')->nullable();
                        // $table->text('feedback_question_values')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('rating_question_options')) {
                    Schema::connection('mysql2')->create('rating_question_options', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->integer('option_id')->nullable();
                        $table->text('option_value')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('rating_question_feedback_details')) {
                    Schema::connection('mysql2')->create('rating_question_feedback_details', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->integer('feedback_question_id')->nullable();
                        $table->text('feedback_question')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('sub_questions')) {
                    Schema::connection('mysql2')->create('sub_questions', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->text('subquestion')->nullable();
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                if (!Schema::connection('mysql2')->hasTable('rating_sub_question_details')) {
                    Schema::connection('mysql2')->create('rating_sub_question_details', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->integer('monitor_id')->nullable();
                        $table->integer('question_id')->nullable();
                        $table->integer('question_type')->nullable();
                        $table->integer('option_id')->nullable();
                        $table->text('option_value')->nullable();
                        $table->integer('sub_question_id')->nullable();
                        $table->text('sub_question')->nullable();
                        $table->date('date')->nullable();
                        $table->time('time')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }

                ///// Edit table entries /////
                if (Schema::connection('mysql2')->hasTable('questions')) {
                    if (!Schema::connection('mysql2')->hasColumn('questions', 'other'))
                    {
                        Schema::connection('mysql2')->table('questions', function (Blueprint $table)
                        {
                            $table->integer('other')->default(0)->after('is_multi_seleted');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('questions', 'order'))
                    {
                        Schema::connection('mysql2')->table('questions', function (Blueprint $table)
                        {
                            $table->integer('order')->default(0)->after('active');
                        });
                    }
                }

                if (Schema::connection('mysql2')->hasTable('rating_question_details')) {
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_details', 'date'))
                    {
                        Schema::connection('mysql2')->table('rating_question_details', function (Blueprint $table)
                        {
                            $table->date('date')->nullable()->before('created_at');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_details', 'time'))
                    {
                        Schema::connection('mysql2')->table('rating_question_details', function (Blueprint $table)
                        {
                            $table->time('time')->nullable()->before('created_at');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_details', 'question_type'))
                    {
                        Schema::connection('mysql2')->table('rating_question_details', function (Blueprint $table)
                        {
                            $table->integer('question_type')->nullable()->after('question_id');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_details', 'monitor_id'))
                    {
                        Schema::connection('mysql2')->table('rating_question_details', function (Blueprint $table)
                        {
                            $table->integer('monitor_id')->nullable()->after('servey_id');
                        });
                    }
                }

                if (Schema::connection('mysql2')->hasTable('rating_question_options')) {
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_options', 'date'))
                    {
                        Schema::connection('mysql2')->table('rating_question_options', function (Blueprint $table)
                        {
                            $table->date('date')->nullable()->before('deleted_at');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_options', 'time'))
                    {
                        Schema::connection('mysql2')->table('rating_question_options', function (Blueprint $table)
                        {
                            $table->time('time')->nullable()->before('deleted_at');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_options', 'question_type'))
                    {
                        Schema::connection('mysql2')->table('rating_question_options', function (Blueprint $table)
                        {
                            $table->integer('question_type')->nullable()->after('question_id');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_options', 'monitor_id'))
                    {
                        Schema::connection('mysql2')->table('rating_question_options', function (Blueprint $table)
                        {
                            $table->integer('monitor_id')->nullable()->after('servey_id');
                        });
                    }
                }

                if (Schema::connection('mysql2')->hasTable('rating_question_feedback_details')) {
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_feedback_details', 'date'))
                    {
                        Schema::connection('mysql2')->table('rating_question_feedback_details', function (Blueprint $table)
                        {
                            $table->date('date')->nullable()->before('deleted_at');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_feedback_details', 'time'))
                    {
                        Schema::connection('mysql2')->table('rating_question_feedback_details', function (Blueprint $table)
                        {
                            $table->time('time')->nullable()->before('deleted_at');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_feedback_details', 'question_type'))
                    {
                        Schema::connection('mysql2')->table('rating_question_feedback_details', function (Blueprint $table)
                        {
                            $table->integer('question_type')->nullable()->after('question_id');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('rating_question_feedback_details', 'monitor_id'))
                    {
                        Schema::connection('mysql2')->table('rating_question_feedback_details', function (Blueprint $table)
                        {
                            $table->integer('monitor_id')->nullable()->after('servey_id');
                        });
                    }
                }

                if (Schema::connection('mysql2')->hasTable('ratings')) {
                    if (!Schema::connection('mysql2')->hasColumn('ratings', 'date'))
                    {
                        Schema::connection('mysql2')->table('ratings', function (Blueprint $table)
                        {
                            $table->date('date')->nullable()->after('performance');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('ratings', 'time'))
                    {
                        Schema::connection('mysql2')->table('ratings', function (Blueprint $table)
                        {
                            $table->time('time')->nullable()->after('date');
                        });
                    }
                    if (!Schema::connection('mysql2')->hasColumn('ratings', 'datetime'))
                    {
                        Schema::connection('mysql2')->table('ratings', function (Blueprint $table)
                        {
                            $table->datetime('datetime')->nullable()->after('time');
                        });
                    }
                }

                ///// Disconnect sql ////
                DB::purge('mysql2');
                DB::disconnect('mysql2'); 
            }
            DB::reconnect();
        }
    }
}
?>