<?php

namespace App\Observers;

use App\Models\Ratings;
use App\Models\HourlyRatings;
use App\Models\HourlyQuestionRatings;
use App\Models\RatingFeedbackQuestionDetails;
use App\Models\RatingQuestionDetails;
use App\Models\RatingQuestionOptions;
use App\Models\RatingSubQuestionOptions;



class RatingsObserver
{
    /**
     * Handle the Ratings "created" event.
     *
     * @param  \App\Models\Ratings  $ratings
     * @return void
     */
    public function created(Ratings $ratings)
    {
        // $date = date('Y-m-d');
        // $time = date('H:i:s');
        $date = $ratings->date;
        $time = $ratings->time;
        $hour = ltrim(date('H'),'0');
        $hour_cl = $hour.'_hr';

        //==== check rating exist or not //////////
        $today_rating = HourlyRatings::select('id','rating',"$hour_cl")
        ->where('monitor_id',$ratings->monitor_id)
        ->whereDate('date',$date)
        //->where(DB::raw("(DATE_FORMAT(time,'%H'))"),$hour)
        ->get()->toArray();
        
        
        $hourly_rating_id='';
        for($i=1;$i<=3;$i++)
        {
            if(!empty($today_rating))
            {
                $hourly_rating_id = array_search($i, array_column($today_rating, 'rating'));
            }
            
            if($i==1) $new_rating = $ratings->positive;
            else if($i==2) $new_rating = $ratings->nuetral;
            else $new_rating = $ratings->negative;

            $p_rating = new HourlyRatings;
            $p_rating['company_id']=$ratings->company_id;
            $p_rating['site_id']=$ratings->site_id;
            $p_rating['monitor_id']=$ratings->monitor_id;
            $p_rating['rating'] = $i;
            $p_rating['time'] = $time;
            
            $p_rating = [
                'monitor_id'=>$ratings->monitor_id,
                'company_id' => $ratings->company_id,
                'site_id' => $ratings->site_id,
                'rating' => $i,
                'date' => $date,
                'time' => $time
            ]; 

            if($hourly_rating_id != '')
            {
                //// update rating /////
                $current_rating = $today_rating[$hourly_rating_id][$hour_cl];
                $p_rating["$hour_cl"] =  $current_rating + $new_rating;
            }else{
                $p_rating["$hour_cl"] = $new_rating;
            }
            //print_r($p_rating);
            HourlyRatings::updateOrCreate(
                ['monitor_id'=>$ratings->monitor_id,'date' => $date,'rating'=>$i],
                $p_rating
            );
        }

        //////// check question ratings /////////
        //// Add rating question details ///////
        if(!empty($ratings->servey_response))
        {
            $servey_response = json_decode($ratings->servey_response);
            if(!empty($servey_response))
            {
                foreach($servey_response as $r)
                {
                    ///// Save Question Details ///////
                    if(!empty($r->response_tone))
                    {
                        $question_details_data['question_id'] = $r->question_id;
                        $question_details_data['question_type'] = $r->question_type;
                        $question_details_data['servey_id'] = $ratings->servey_id;
                        $question_details_data['monitor_id'] = $ratings->monitor_id;
                        $question_details_data['rating'] = !empty($r->rating) ? $r->rating : NULL;
                        $question_details_data['response_tone'] = $r->response_tone;
                        $question_details_data['date'] = $date;
                        $question_details_data['time'] = $time;
                        $question_details_data['created_at'] = date('Y-m-d H:i:s');
                        $question_details_data['updated_at'] = date('Y-m-d H:i:s');
                        $que_details_id = RatingQuestionDetails::insert($question_details_data);
                    }

                    //// Save rating feedback details ///////
                    if(!empty($r->feedback_question_ids))
                    {
                        $fds = $r->feedback_question_ids;
                        $fd_values = $r->feedback_question_values;

                        $feedback_que_data=[];
                        foreach($fds as $k=>$f)
                        {
                            $feedback_que_data[] = [
                                'question_id' => $r->question_id,
                                'question_type' => $r->question_type,
                                'servey_id' => $ratings->servey_id,
                                'monitor_id' => $ratings->monitor_id,
                                'feedback_question_id' => $f,
                                'feedback_question' => $fd_values[$k],
                                'date' => $date,
                                'time' => $time,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                        RatingFeedbackQuestionDetails::insert($feedback_que_data);
                    }

                    //// Save rating question options ///////
                    if(!empty($r->option_ids))
                    {
                        $opts = $r->option_ids;
                        $opt_values = $r->options_value;

                        $option_data=[];
                        foreach($opts as $o=>$v)
                        {
                            $option_data[] = [
                                'question_id' => $r->question_id,
                                'question_type' => $r->question_type,
                                'servey_id' => $ratings->servey_id,
                                'monitor_id' => $ratings->monitor_id,
                                'option_id' => $v,
                                'option_value' => $opt_values[$o],
                                'date' => $date,
                                'time' => $time,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                        RatingQuestionOptions::insert($option_data);
                    }

                    //// Save rating sub questions details ///////
                    if(!empty($r->sub_questions))
                    {
                        $sub_questions_data=[];
                        foreach($r->sub_questions as $sq)
                        {
                            $sub_questions_data[] = [
                                'question_id' => $r->question_id,
                                'question_type' => $r->question_type,
                                'servey_id' => $ratings->servey_id,
                                'monitor_id' => $ratings->monitor_id,
                                'sub_question_id' => $sq->id,
                                'sub_question' => $sq->sub_question,
                                'option_id' => $sq->option_id,
                                'option_value' => $sq->options_value,
                                'date' => $date,
                                'time' => $time,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                        if(!empty($sub_questions_data))
                        {
                            RatingSubQuestionOptions::insert($sub_questions_data);
                        }
                    }

                    //==== Save rating question hourly //////////
                    if(!empty($r->response_tone) || $r->response_tone!=0)
                    {
                        $today_hourly_rating = HourlyQuestionRatings::select('id','rating',"$hour_cl")
                        ->where('monitor_id',$ratings->monitor_id)
                        ->where('question_id',$r->question_id)
                        ->where('rating',$r->response_tone)
                        ->whereDate('date',$date)
                        ->get()->first();
    
                        if(!empty($today_hourly_rating))
                        {
                            $hourly_rating = $today_hourly_rating->$hour_cl + 1;
                        }else{
                            $hourly_rating = 1;
                        }
    
                        $newQhrating = HourlyQuestionRatings::updateOrCreate([
                            'monitor_id'   => $ratings->monitor_id,
                            'question_id'   => $r->question_id,
                            'rating'   => $r->response_tone,
                            'date'   => $date,
                        ],[
                            'monitor_id'=> $ratings->monitor_id,
                            'question_id' => $r->question_id,
                            'site_id' => $ratings->site_id,
                            'servey_id'=> $ratings->servey_id,
                            'rating' => $r->response_tone,
                            "$hour_cl" => $hourly_rating,
                            'date' => $date,
                            'time' => $time
                        ]);
                    }
                    
                }
            }
        }
    }

    /**
     * Handle the Ratings "updated" event.
     *
     * @param  \App\Models\Ratings  $ratings
     * @return void
     */
    public function updated(Ratings $ratings)
    {
        //
    }

    /**
     * Handle the Ratings "deleted" event.
     *
     * @param  \App\Models\Ratings  $ratings
     * @return void
     */
    public function deleted(Ratings $ratings)
    {
        //
    }

    /**
     * Handle the Ratings "restored" event.
     *
     * @param  \App\Models\Ratings  $ratings
     * @return void
     */
    public function restored(Ratings $ratings)
    {
        //
    }

    /**
     * Handle the Ratings "force deleted" event.
     *
     * @param  \App\Models\Ratings  $ratings
     * @return void
     */
    public function forceDeleted(Ratings $ratings)
    {
        //
    }
}
