<?php

namespace App\Http\Controllers;

use App\Http\Controllers\RoleController;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Requests\Admin\CompanyRequest;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use Cviebrock\EloquentSluggable\Services\SlugService;
use Config;
use DB;

class CompanyController extends RoleController
{
    protected static $title = 'Companies';

    
    public function index()
    {
        //print_r(auth()->user());
        $companies = User::where('role','2')->paginate(Config::get('constants.page_limit'));
        return view('admin.companies.index',['companies'=>$companies,'title'=>static::$title]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.companies.create',['title'=>static::$title]);
    }

    public function store(CompanyRequest $request)
    {

        /// Upload logo /////
        $profile_photo_path='';
        if($request->file('profile_photo_path')){
            $image_path = 'images/company_images';
            $profile_photo_path = UploadFile($request->file('profile_photo_path'),$image_path);
            // $path = $request->file('profile_photo_path');
            // //$image = $path->getClientOriginalName();
            // //$image = time().'_'.$path->getClientOriginalName();
            // $image = time().rand(1,99).'.'.$path->extension();
            // $path->move(public_path('images/company_images'), $image);
            // $profile_photo_path = 'images/company_images/'.$image;
        }

        $slug = SlugService::createSlug(User::class, 'slug', $request->name);
        $db_name = str_replace('-','_',$slug);

        $company = User::create([
            'role' => 2,
            'user_group'=>'admin',
            'username' => $request->username,
            'name' => $request->name,
            'slug' => $slug,
            'email' => $request->email,
            'contact_no' => $request->contact_no,
            'password' => Hash::make($request->password),
            'profile_photo_path'=> (!empty($profile_photo_path)) ? $profile_photo_path : '',
            'db_name'=>$db_name
        ]);
        if($company)
        {
            
            //// Create new db 
            $preDatabase = Config::get('database.connections.mysql.database');
            DB::statement("CREATE DATABASE IF NOT EXISTS $db_name");
            // $new = Config::set('database.connections.mysql.database', $db_name);
            // DB::purge('mysql');
    
            // DB::reconnect('mysql');
    
            //\Artisan::call('migrate');
            // Config::set('database.connections.mysql.database', $preDatabase);
            return redirect()->route('companies.index')->with('message', 'Company added successfully.');
        }

        return redirect()->back()
        ->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        echo '1';
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Company  $company
     * @return \Illuminate\Http\Response
    */
    public function edit($id)
    {
        if($id)
        {
            $company = User::find($id);

            if($company)
            {
                return view('admin.companies.edit', ['company' => $company,'title'=>static::$title,'is_edit'=>1]);
            }
        }

        abort(404);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function update(CompanyRequest $request, $id)
    {
        if($id)
        {
            $post = $request->all();
            $post['updated_by'] = Auth::user()->id;

            /// Upload logo /////
            if($request->file('profile_photo_path')){
                // $path = $request->file('profile_photo_path');
                // //$image = time().'_'.$path->getClientOriginalName();
                // $image = time().rand(1,99).'.'.$path->extension();
                // $path->move(public_path('images/company_images'), $image);
                // $profile_photo_path = 'images/company_images/'.$image;
                // $post['profile_photo_path'] = $profile_photo_path;

                $image_path = 'images/company_images';
                $post['profile_photo_path'] = UploadFile($request->file('profile_photo_path'),$image_path);

            }
            
            $company = User::find($id);

            if($post && $company)
            {
                if($company->update($post))
                {
                    return redirect()->route('companies.index')->with('message', 'Company edited successfully.');
                }
            }

            return redirect()->back()->with('error', __('messages.somethingWrong'));
        }

        abort(404);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $up_data['deleted_by'] = Auth::user()->id;
        User::find($id)->update($up_data);

        //User::find($id)->delete();
  
        return redirect()->back()->with('message', 'Company deleted successfully.');
    }

    public function restoreAll()
    {
        User::onlyTrashed()->restore();
  
        return redirect()->back();
    }
}
