<?php

namespace App\Http\Controllers;

use App\Models\Servey;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\Admin\ServeyRequest;
use Cviebrock\EloquentSluggable\Services\SlugService;
use Illuminate\Support\Facades\Schema;
use App\Http\Controllers\CompanyRoleController;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Artisan;



class ServeyController extends CompanyRoleController
{
    protected static $title = 'Servey';

    // Image types
     protected static $image_types = [
        '1' => 'Portrait',
        '2' => 'Landscape'
    ]; 

    
    // Image size
    protected static $image_sizes = [
        '31' => '31%',
        '32' => '32%',
        '33' => '33%',
        '34' => '34%',
        '35' => '35%',
        '36' => '36%',
        '37' => '37%',
        '38' => '38%',
        '39' => '39%',
        '40' => '40%',
        '41' => '41%',
        '42' => '42%',
        '43' => '43%',
        '44' => '44%',
        '45' => '45%',
        '46' => '46%',
        '47' => '47%',
        '48' => '48%',
        '49' => '49%',
        '50' => '50%'
    ]; 

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $servey=array();
        if (Schema::connection('mysql2')->hasTable('servey')) {
            $servey = Servey::paginate(Config::get('constants.page_limit'));
        }
        return view('admin.servey.index',['servey'=>$servey,'title'=>static::$title]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.servey.create',['image_types'=>static::$image_types,'image_sizes'=>static::$image_sizes,'title'=>static::$title]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(ServeyRequest $request)
    {
        //echo '<pre>';print_r($request->all());echo '</pre>';
        $slug = SlugService::createSlug(Servey::class, 'slug', $request->name);
        $servey = Servey::create([
            'name' => $request->name,
            'size' => !empty($request->size) ? $request->size : '',
            'type' => isset($request->type) ? $request->type : '',
            'message' => !empty($request->message) ? $request->message : '',
            'ask_customer_details' => isset($request->ask_customer_details) ? 1 : 0,
            'slug' => $slug,
            'created_by' => Auth::user()->id
        ]);
        if($servey)
        {
            //// Save cofiguarations //////
            
            ////// save date configuaration//////
            $date_config = \App\Models\ServeyConfig::create([
                'servey_id'=>$servey->id,
                'config_id' => 1,
                'date_format' => $request->date_format,
                'created_by' => Auth::user()->id
            ]); 
            
            ////// save clock configuaration//////
            $clock_config = \App\Models\ServeyConfig::create([
                'servey_id'=>$servey->id,
                'config_id' => 2,
                'clock_format' => $request->clock_format,
                'created_by' => Auth::user()->id
            ]); 
           
           
            /// Upload advertisement /////
            if(isset($request->items) && !empty($request->items))
            {
                $advert_data = [];
                $image_path = 'images/advertisements';
                foreach ($request->items as $key => $items)
                {
                    if(isset($items['image']))
                    {
                        $advert_image = UploadFile($items['image'],$image_path);
                        if (file_exists(public_path($advert_image))) {
                            //// save advertise /////
                            $advert1 = \App\Models\ServeyAdvert::create([
                                'servey_id'=>$servey->id,
                                'image' => $advert_image,
                                'sort'=>$key+1,
                                'created_by' => Auth::user()->id
                            ]); 
                        }
                    }
                }
            }
            return redirect()->route('servey.index')->with('message', 'Servey added successfully.');
        }

        return redirect()->back()
        ->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    // public function show(Servey $servey)
    // {
    //     echo 'f';exit;
    // }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        if($id)
        {
            $servey = Servey::with('advertisements')->find($id);

            if($servey)
            {
                $monitor_config = \App\Models\ServeyConfig::where('servey_id',$id)->get()->toArray();
                //echo '<pre>';print_r($monitor_config);echo '</pre>';
                $date_key = array_search('1', array_column($monitor_config, 'config_id'));
                $clock_key = array_search('2', array_column($monitor_config, 'config_id'));
                $advert_key = array_search('3', array_column($monitor_config, 'config_id'));
                
                $config['date_format']=$config['clock_format']=$config['advertise']='';
                $config['is_advert']=0;
                if($date_key!='')
                {
                    $config['date_format']=$monitor_config[$date_key]['date_format'];
                }
                if($clock_key!='')
                {
                    $config['clock_format']=$monitor_config[$clock_key]['clock_format'];
                }
                if($advert_key!='')
                {
                    $config['is_advert']=1;
                    $config['advertise']=\App\Models\ServeyAdvert::select('image')->where('servey_id',$id)->get()->first();
                }
                return view('admin.servey.edit', ['servey' => $servey,'config'=>$config,'image_sizes'=>static::$image_sizes,'image_types'=>static::$image_types,'title'=>static::$title,'is_edit'=>1]);
            }
        }

        abort(404);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function update(ServeyRequest $request, $id)
    {
        if($id)
        {
            $post = $request->all();
            $post['ask_customer_details'] = isset($request->ask_customer_details) ? 1 : 0;
            $post['updated_by'] = Auth::user()->id;
            $servey = Servey::with('advertisements')->find($id);

            if($post && $servey)
            {
                if($servey->update($post))
                {
                    ////// save date configuaration//////
                    $date_config = [
                        'monitor_id'=>$id,
                        'config_id' => 1,
                        'date_format' => $request->date_format,
                        'updated_by' => Auth::user()->id,
                        'updated_at' => date('Y-m-d H:i:s')
                    ]; 
                    \App\Models\ServeyConfig::updateOrCreate(
                        ['servey_id' => $id,'config_id'=>1],
                        $date_config
                    );
                    
                    ////// save clock configuaration//////
                    $clock_config = [
                        'servey_id'=>$id,
                        'config_id' => 2,
                        'clock_format' => $request->clock_format,
                        'updated_by' => Auth::user()->id,
                        'updated_at' => date('Y-m-d H:i:s')
                    ]; 
                    \App\Models\ServeyConfig::updateOrCreate(
                        ['servey_id' => $id,'config_id'=>2],
                        $clock_config
                    );

                    /// Upload advertisement /////
                    $advertisement_ids = $advertise_uploads = [];
                    if(!empty($servey->advertisements))
                    {
                        foreach($servey->advertisements as $a)
                        {
                            $advertisement_ids[] = $a->id;
                        }
                    }
                    if(isset($request->items) && !empty($request->items))
                    {
                        $advert_data = $keep_items = $deleted_ids = [];
                        $image_path = 'images/advertisements';
                        foreach ($request->items as $key => $items)
                        {
                            if(isset($items['image']))
                            {
                                $advert_image = UploadFile($items['image'],$image_path);
                                if (file_exists(public_path($advert_image))) {
                                    //// save advertise /////
                                    $advert_data = [
                                        'servey_id'=>$servey->id,
                                        'image' => $advert_image,
                                        'sort'=>$key+1,
                                        'created_by' => Auth::user()->id
                                    ]; 
                                }
                                $items['advertisement_id'] = (isset($items['advertisement_id']) && !empty($items['advertisement_id'])) ? $items['advertisement_id'] : 0;
                                $poItem = \App\Models\ServeyAdvert::updateOrCreate([
                                    'id' => $items['advertisement_id']
                                ], $advert_data);
                            }
                            $keep_items[] = $items['advertisement_id'];
                        }
                        if(!empty($advertisement_ids) && !empty($keep_items))
                        {
                            $deleted_ids =  array_diff($advertisement_ids, $keep_items);
                        }
                        
                        if(!empty($deleted_ids))
                        {
                            $advertise_uploads = \App\Models\ServeyAdvert::select('image')->where('servey_id', $servey->id)->whereIn('id', $deleted_ids)->get()->toArray();
                            ///// Unlink from uploads /////
                            if(!empty($advertise_uploads))
                            {
                                foreach($advertise_uploads as $img)
                                {
                                    DeleteFile($img['image']);
                                }
                            }
                            ///// deleted from database ///////
                           $delete = \App\Models\ServeyAdvert::where('servey_id', $servey->id)->whereIn('id', $deleted_ids)->delete();
                        }
                    }

                    return redirect()->route('servey.index')->with('message', 'Servey edited successfully.');
                }
            }

            return redirect()->back()->with('error', __('messages.somethingWrong'));
        }

        abort(404);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $up_data['deleted_by'] = Auth::user()->id;
        Servey::find($id)->update($up_data);

        Servey::find($id)->delete();
  
        return redirect()->back()->with('message', 'Servey deleted successfully.');
    }
}
