<?php

namespace App\Http\Controllers;

use App\Models\Servey;
use App\Models\Questions;
use App\Models\QuestionOptions;
use App\Models\TemplateFeedbackQuestions;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\Admin\ServeyTemplateRequest;
use Illuminate\Support\Facades\Schema;
use App\Http\Controllers\CompanyRoleController;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Artisan;
use App\Models\FeedbackQuestion;
use Cviebrock\EloquentSluggable\Services\SlugService;



class ServeyTemplateController extends CompanyRoleController
{
    protected static $title = 'Questions';
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
    */

     // Question types
     protected static $question_types = [
        '1' => 'Smiley',
        '2' => 'Rating Bar',
        '3' => 'Multi Selection',
        '4' => 'Scale',
        '5' => 'Multi Options',
        '6' => 'Yes/No Question',
        '7' => 'Free Text'
    ]; 

    public function view($servey_id)
    {
        $servey='';
        if (Schema::connection('mysql2')->hasTable('servey')) {
            $servey = Servey::where('id',$servey_id)->first();
        }
        $questions=array();
        if (Schema::connection('mysql2')->hasTable('questions')) {
            $questions = Questions::select('id','question','description')->where('servey_id',$servey_id)->get()->toArray();
        }
        return view('admin.servey.templates.index')->with(compact('servey','questions'));
        return view('admin.servey.templates.index',['servey'=>$servey,'questions'=>$questions,'question_types'=>static::$question_types,'title'=>static::$title]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create($servey_id)
    {
        $servey='';$feedback_questions=$positive_feedback_questions=$negative_feedback_questions=array();
        if (Schema::connection('mysql2')->hasTable('servey')) {
            $servey = Servey::where('id',$servey_id)->first();
            
            ///==== Start to Get Feedback Questions ====///////
            if (Schema::connection('mysql2')->hasTable('feedback_questions')) {
                $feedback_questions = FeedbackQuestion::where('active',1)->select('question','id','question_type')->get()->toArray();
                if(!empty($feedback_questions))
                {
                    foreach($feedback_questions as $f)
                    {
                        // ==== Get Postive Questions ====///
                        if($f['question_type']==1){
                            $positive_feedback_questions[$f['id']]=$f['question'];
                        }

                        // ==== Get Negative Questions ====///
                        if($f['question_type']==2){
                            $negative_feedback_questions[$f['id']]=$f['question'];
                        }
                    }
                }
                ///==== End to Get Feedback Questions ====///////
            }
        }
        return view('admin.servey.templates.create',['servey'=>$servey,'positive_feedback_questions'=>$positive_feedback_questions,'negative_feedback_questions'=>$negative_feedback_questions,'question_types'=>static::$question_types,'title'=>static::$title]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(ServeyTemplateRequest $request,$servey_id)
    {
        //echo '<pre>';print_r($_POST);echo '</pre>';exit;
        $question = Questions::create([
            'servey_id' => $servey_id,
            'question' => $request->question,
            'description' => $request->description,
            'positive_title' => isset($request->positive_title) ? $request->positive_title : '',
            'negative_title' => isset($request->negative_title) ? $request->negative_title : '',
            'question_type' => $request->question_type,
            'is_multi_seleted' => ($request->question_type==3 || $request->question_type==5) ? 1 : 0,
            'icon_type' => isset($request->image_type) ? $request->image_type : 0,
            'created_by' => Auth::user()->id,
        ]);
        if($question)
        {

            ////// Save multiple Options //////
            // ==== Allow only Question Type = Multi Options ======/
            if($request->question_type==5)
            {
                $que_options_datas=array();
                for($fq=1;$fq<=4;$fq++)
                {
                    echo $option_val = 'option'.$fq;
                    $status_val = 'response_tone'.$fq;
                    $option = $request->$option_val;
                    if(isset($option) && !empty($option))
                    {
                        $status =$request->$status_val;
                        $que_options_datas[] = [
                            'question_id' => $question->id,
                            'option_id' => $fq,
                            'options' => $option,
                            'response_tone' => !empty($status) ? $status : 1,
                            'created_by' => Auth::user()->id,
                            'created_at' => date('Y-m-d H:i:s'),
                            'updated_at' => date('Y-m-d H:i:s')
                        ];
                    }
                }
                if(!empty($que_options_datas)) QuestionOptions::insert($que_options_datas);
            }

            ///// Save Positive/Negative Questions /////
            // ==== Allow only Question Type = Smiley & Rating Bar ======/
            if($request->question_type==1 || $request->question_type==2)
            {
                ///// Add Positive Feedback Questions //////
                if(isset($request->postive_questions) && !empty($request->postive_questions))
                {
                    $positive_questions=array();
                    foreach($request->postive_questions as $p)
                    {
                        if(!empty($p))
                        {
                            $positive_questions[] = [
                                'servey_id' => $servey_id,
                                'question_id' => $question->id,
                                'feedback_question_id' => $p,
                                'response_tone' => 1,
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                    }
                    if(!empty($positive_questions)) TemplateFeedbackQuestions::insert($positive_questions);
                }

                ///// Add Negative Feedback Questions //////
                if(isset($request->negative_questions) && !empty($request->negative_questions))
                {
                    $negative_questions=array();
                    foreach($request->negative_questions as $n)
                    {
                        if(!empty($n))
                        {
                            $negative_questions[] = [
                                'servey_id' => $servey_id,
                                'question_id' => $question->id,
                                'feedback_question_id' => $n,
                                'response_tone' => 2,
                                'created_by' => Auth::user()->id,
                                'created_at' => date('Y-m-d H:i:s'),
                                'updated_at' => date('Y-m-d H:i:s')
                            ];
                        }
                    }
                    if(!empty($negative_questions)) TemplateFeedbackQuestions::insert($negative_questions);
                }
            }
            
            ///// Save Multiple images with +/- /////
            // ==== Allow only Question Type = Multi Selection ======/
            if($request->question_type==3)
            {
                $option_img_data=array();
                for($img=1;$img<=4;$img++)
                {
                    if($request->file("option$img")){
                        
                        $status_val = 'response_tone'.$img;
                        $status =$request->$status_val;

                        $path = $request->file("option$img");
                        $image = time().'_'.$path->getClientOriginalName();
                        $path->move(public_path('images/templates/'), $image);

                        $option_img_data[] = [
                            'question_id' => $question->id,
                            'option_id' => $img,
                            'options' => 'images/templates/'.$image,
                            'response_tone' => !empty($status) ? $status : 1,
                            'created_by' => Auth::user()->id,
                            'created_at' => date('Y-m-d H:i:s'),
                            'updated_at' => date('Y-m-d H:i:s')
                        ];
                    }
                }

                //// Insert Image options /////
                if(!empty($option_img_data)) QuestionOptions::insert($option_img_data);
            }

            return redirect()->route('servey.template',[$servey_id])->with('message', 'Template added successfully.');
        }

        return redirect()->back()
        ->withInput();
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    // public function show(Servey $servey)
    // {
    //     echo 'f';exit;
    // }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        if($id)
        {
            $servey = Servey::find($id);

            if($servey)
            {
                return view('admin.servey.edit', ['servey' => $servey,'title'=>static::$title,'is_edit'=>1]);
            }
        }

        abort(404);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function update(ServeyRequest $request, $id)
    {
        if($id)
        {
            $post = $request->all();
            $post['updated_by'] = Auth::user()->id;
            $servey = Servey::find($id);

            if($post && $servey)
            {
                if($servey->update($post))
                {
                    return redirect()->route('servey.index')->with('message', 'Servey edited successfully.');
                }
            }

            return redirect()->back()->with('error', __('messages.somethingWrong'));
        }

        abort(404);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Servey  $servey
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $up_data['deleted_by'] = Auth::user()->id;
        Servey::find($id)->update($up_data);

        Servey::find($id)->delete();
  
        return redirect()->back()->with('message', 'Servey deleted successfully.');
    
    }

    /* Duplicate sevey */
    public function duplicate($id)
    {
        if($id)
        {
            $servey = Servey::find($id);
            if($servey)
            {
                $slug = SlugService::createSlug(Servey::class, 'slug', $servey->name);
                $d_servey = Servey::create([
                    'name' => $servey->name,
                    'slug' => $slug,
                    'created_by' => Auth::user()->id
                ]);

                if($d_servey)
                {
                    ///// Get servey Questions //////
                    $questions = Questions::where('servey_id',$id)->get();
                    $que_data=$question_option_data=$feedback_que_data='';
                    foreach($questions as $question)
                    {
                        $que_data = $question->replicate();
                        $que_data->created_by = Auth::user()->id;
                        $que_data->servey_id = $d_servey->id;
                        $que_data->created_at = now();
                        $duplicate_question = $que_data->save();
                        
                        if($duplicate_question)
                        {
                            $question_options='';
                            $question_option_data='';
                            ////// Insert Question Options /////////
                            $question_options = QuestionOptions::where('question_id',$question->id)->get();
                            if(!empty($question_options))
                            {
                                foreach($question_options as $opt)
                                {
                                    $question_option_data = $opt->replicate();
                                    $question_option_data->question_id = $que_data->id;
                                    $question_option_data->created_at = now();
                                    $question_option_data->save();
                                }
                            }
                            $feedback_que_data='';
                            /////// Insert +/- questions if any //////
                            $feedback_que = TemplateFeedbackQuestions::where('question_id',$question->id)->get();
                            if(!empty($feedback_que))
                            {
                                foreach($feedback_que as $f)
                                {
                                    $feedback_que_data = $f->replicate();
                                    $feedback_que_data->question_id = $que_data->id;
                                    $feedback_que_data->servey_id = $d_servey->id;
                                    $feedback_que_data->created_at = now();
                                    $feedback_que_data->save();
                                }
                            }
                        }
                    }
                    return redirect()->route('servey.index')->with('message', 'Servey duplicated successfully.');
                }
            }
        }
        abort(404);
    }
}
