<?php

namespace App\Http\Controllers\api;

use App\Models\User; 
use App\Models\Monitor; 
use App\Models\Token;
use App\Models\MonitorDeviceTokens;
use App\Models\Servey;
use App\Models\Questions;
use App\Models\QuestionOptions;
use App\Models\FeedbackQuestion;
use App\Models\TemplateFeedbackQuestions;
use App\Models\RatingCustomerDetails;
use App\Models\RatingFeedbackQuestionDetails;
use App\Models\RatingQuestionDetails;
use App\Models\RatingQuestionOptions;
use App\Models\SubQuestions;
use Validator;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Str; 
use App\api_helper;
use Config;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\Schema\Blueprint;

class MonitorApi extends Controller
{
    //// Just for Testing purpose //////
    function validateToken(Request $request)
    {
        echo bearerToken($request)->token;
        //MethodAllow('post',$request->method);
    }

    function GetSettings(Request $request)
    {
        $request_parameters = array("company_id","site_id","monitor_id","servey_id");
        $token = json_decode(bearerToken($request,$request_parameters));
        if($token->is_error==1)
        {
            return error((object)null,'Missing Parameter '.$token->data,400);
        }

        /// take request params /////
        $servey_id = $request->servey_id;
        $company_id = $request->company_id;
        $monitor_id = $request->monitor_id;
        $site_id = $request->site_id;

        //// Get company profile pic ///////
        $company_details = User::select('profile_photo_path')->where('id',$company_id)->get()->first();
        $data['profile_pic'] = !empty($company_details->profile_photo_path) ? url($company_details->profile_photo_path) : '';
        ///// Get Advertise ///////
        $data['advertisement_image']='';
        if (Schema::connection('mysql2')->hasTable('monitor_advertise')) {
            $monitor_advertise = \App\Models\MonitorAdvert::select('image')->where('monitor_id',$monitor_id)->get()->first();
            
            if(!empty($monitor_advertise))
            {
                $data['advertisement_image'] = url($monitor_advertise->image);
            }
        }
        $data['advertisements']=array();
        $advert_images = [];
        if (Schema::connection('mysql2')->hasTable('servey_advertise')) {
            $servey_advertise = \App\Models\ServeyAdvert::select('image')->where('servey_id',$servey_id)->get()->toArray();
            $data['advertisements'] = $servey_advertise;
            if(!empty($servey_advertise))
            {
                foreach($servey_advertise as $k=>$s)
                {
                    $advert_images[] = url($s['image']);
                }
            }
        }
        $data['advertisements']=$advert_images;
        
        //// Get Monitor configurations /////
        $data['config']=array();
        if (Schema::connection('mysql2')->hasTable('monitor_config')) {
            $monitor_config = \App\Models\ServeyConfig::select('config_id','date_format','clock_format')->where('servey_id',$servey_id)->get()->toArray();
            
            if(!empty($monitor_config))
            {
                $ci=0;
                foreach($monitor_config as $key=>$c)
                {
                    $config_title = \App\Models\Config::select('title')->where('id',$c['config_id'])->get()->first();
                    $conf_title = !empty($config_title->title) ? $config_title->title : '';
                    $data['config'][$ci]['config_id']=$c['config_id'];
                    $data['config'][$ci]['title']=$conf_title;
                    $data['config'][$ci]['date_format']=($c['config_id']==1) ? (($c['date_format']==1) ? 'dd/mm/yy' : 'mm/dd/yy') : '';
                    $data['config'][$ci]['clock_format']= ($c['config_id']==2) ? (($c['clock_format']==1) ? 12 : 24) : '';
                    $ci++;
                }
            }
        }

        //// Get Servey Details /////
        $data['servey_details']=Servey::select('id','name','type','size','ask_customer_details','message')->where('id',$servey_id)->get()->first();

        $question_list = Questions::with('options','servey_feedback_questions','sub_questions')->where('servey_id',$servey_id)->orderBy('order','ASC')->get()->toArray();

        if(!empty($question_list))
        {
            $i=0;
            foreach($question_list as $r)
            {
                $data['questions_data'][$i]['id']=$r['id'];
                $data['questions_data'][$i]['servey_id']=$r['servey_id'];
                $data['questions_data'][$i]['question']=$r['question'];
                $data['questions_data'][$i]['order']=$r['order'];
                $data['questions_data'][$i]['description']=$r['description'];
                $data['questions_data'][$i]['positive_title']=$r['positive_title'];
                $data['questions_data'][$i]['negative_title']=$r['negative_title'];
                $data['questions_data'][$i]['question_type']=$r['question_type'];
                $data['questions_data'][$i]['icon_type']=$r['icon_type'];
                $data['questions_data'][$i]['is_multi_seleted']=$r['is_multi_seleted'];
                $data['questions_data'][$i]['other']=$r['other'];

                ///// Get Question Options : if question has multi option //////////////////
                $data['questions_data'][$i]['options_data'] = $r['options'];

                ///// Get Sun=b Questions : if question has multi questions //////////////////
                $data['questions_data'][$i]['sub_questions_data'] = $r['sub_questions'];

                //$data['questions_data'][$i]['servey_feedback_questions'] = $r['servey_feedback_questions'];
                //// Get +/- questions list ///////
                if(!empty($r['servey_feedback_questions']))
                {
                    $j=0;
                    foreach($r['servey_feedback_questions'] as $fd)
                    {
                        $data['questions_data'][$i]['feedback_questions'][$j]['id'] = $fd['id'];
                        $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question_id'] = $fd['feedback_question_id'];
                        $data['questions_data'][$i]['feedback_questions'][$j]['response_tone'] = $fd['response_tone'];
                        if(!empty($fd['feedback_question_id']))
                        {
                            $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question'] = FeedbackQuestion::select('question','question_type','issue_type')->where('id',$fd['feedback_question_id'])->first();
                        }else{
                            $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['question']='Other';
                            $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['question_type']=($fd['response_tone']==1) ? 1 : 2;
                            $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['issue_type']=0;
                        }
                        
                        $j++;
                    }

                    // $data['questions_data'][$i]['feedback_questions'][$j]['id']=0;                  
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question_id']=0;                  
                    // $data['questions_data'][$i]['feedback_questions'][$j]['response_tone']=0;                  
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['question']='Other';                   
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['question_type']=0;                   
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['issue_type']=0;                  
 

                }else{
                    // $j=0;

                    // $data['questions_data'][$i]['feedback_questions'][$j]['id']=0;                  
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question_id']=0;                  
                    // $data['questions_data'][$i]['feedback_questions'][$j]['response_tone']=0;                  
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['question']='Other';                   
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['question_type']=0;                   
                    // $data['questions_data'][$i]['feedback_questions'][$j]['feedback_question']['issue_type']=0;    

                    //$data['questions_data'][$i]['feedback_questions'] = array();
                }
                $i++;
            }
        }
        return success($data,'Success',200);
    }


    //// Save Feedback //////
    function SaveFeedback(Request $request)
    {
        $request_parameters = array("company_id","site_id","monitor_id","servey_id","servey_response");
        $token = json_decode(bearerToken($request,$request_parameters));
        if($token->is_error==1)
        {
            return error((object)null,'Missing Parameter '.$token->data,400);
        }
        $servey_response = json_decode($request->servey_response);
        $final=array();
        $positive_tone_count=$negative_tone_count=$neutral_tone_count=0;
        if(!empty($servey_response))
        {

            $raings = new \App\Models\Ratings;
            $raings['company_id']=$request->company_id;
            $raings['site_id']=$request->site_id;
            $raings['monitor_id']=$request->monitor_id;
            $raings['servey_id']=$request->servey_id;
            if(isset($request->date) && !empty($request->date))
            {
                $raings['date']=$request->date;
            }else{
                $raings['date']=date('Y-m-d');
            }

            if(isset($request->time) && !empty($request->time))
            {
                $raings['time']=$request->time;
            }else{
                $raings['time']=date('H:i:s');
            }

            $raings['datetime']=$request->date.' '.$request->time;

            $question_count=count($servey_response);
            foreach($servey_response as $r)
            {
                //print_r($r);
                //// Question Types ///////
                ///// Type : 1 -> Smiley //////
                ///// Type : 2 -> Rating bar //////
                ///// Type : 3 -> Multi selection //////
                ///// Type : 4 -> Scale //////
                ///// Type : 5 -> Multi Options //////
                ///// Type : 6 -> Yes/No //////
                ///// Type : 7 -> Free text //////
                ///// Response tone 1 : Positive, 2:Netrual, 3:Negative //////
                if($r->question_type==1)
                {
                    ///// Smiley //////
                    if($r->rating < 3)
                    {
                        $response_tone=3;
                        $negative_tone_count += 1;
                    }else if($r->rating == 3){
                        $response_tone=2;
                        $neutral_tone_count += 1;
                    }else if($r->rating == 4){
                        $response_tone=1;
                        $positive_tone_count += 1;
                    }                    
                }else if($r->question_type==2){
                    ///// rating bar //////
                    if($r->rating < 3)
                    {
                        $response_tone=3;
                        $negative_tone_count += 1;
                    }else if($r->rating == 3){
                        $response_tone=2;
                        $neutral_tone_count += 1;
                    }else if($r->rating >= 4){
                        $response_tone=1;
                        $positive_tone_count += 1;
                    }     
                }else if($r->question_type==4){
                    ///// Scale ///////
                    $icon_type = Questions::select('icon_type')->where('id',$r->question_id)->get()->first()->icon_type;
                    //// Icon type : 1,3,5 = 5 scale 2,4,6 = 10 scale ///// 
                    if($icon_type==1 || $icon_type==3 || $icon_type==5)
                    {
                        if($r->rating < 3)
                        {
                            $response_tone=3;
                            $negative_tone_count += 1;
                        }else if($r->rating == 3){
                            $response_tone=2;
                            $neutral_tone_count += 1;
                        }else if($r->rating >= 4){
                            $response_tone=1;
                            $positive_tone_count += 1;
                        }     
                    }
                    else if($icon_type==2 || $icon_type==4 || $icon_type==6){
                        if($r->rating < 5)
                        {
                            $response_tone=3;
                            $negative_tone_count += 1;
                        }else if($r->rating == 5 || $r->rating == 6){
                            $response_tone=2;
                            $neutral_tone_count += 1;
                        }else if($r->rating >= 7){
                            $response_tone=1;
                            $positive_tone_count += 1;
                        }     
                    }
                }else if($r->question_type==6){
                    //// Yes/No //////
                    $yes_no_response_tone = QuestionOptions::select('response_tone')->where('question_id',$r->question_id)->where('option_id',$r->rating)->first()->response_tone;
                    if($yes_no_response_tone==1){
                        $response_tone=1;
                        $positive_tone_count += 1;
                    }else{
                        $response_tone=3;
                        $negative_tone_count += 1;
                    }
                }
                else if($r->question_type==3){
                    //// Multi Selection //////
                    $multi_options = $r->option_ids;
                    if(!empty($multi_options)){
                        // if($multi_options == null){
                        //     // There was an error
                        //     return error((object) null, 'Bad JSON received', 400);
                        // }  
                        $positive_opt_count=$negative_opt_count=0;                  
                        foreach($multi_options as $mo)
                        {
                            $multi_opt_response_tone = QuestionOptions::select('response_tone')->where('question_id',$r->question_id)->where('id',$mo)->first()->response_tone;
                            if($multi_opt_response_tone==1){
                                $response_tone=1;
                                $positive_opt_count++;
                            }else{
                                $response_tone=3;
                                $negative_opt_count++;
                            }
                        }
                        if($positive_opt_count > $negative_opt_count)
                        {
                            $response_tone=1;
                            $positive_tone_count += 1;
                        }else if($positive_opt_count == $negative_opt_count){
                            $response_tone=2;
                            $neutral_tone_count += 1;
                        }else if($positive_opt_count < $negative_opt_count){
                            $response_tone=3;
                            $negative_tone_count += 1;
                        }
                    }
                }
                else if($r->question_type==5 || $r->question_type==8){
                    //// Multi options //////
                    //print_r($r->option_ids);exit;
                    //$multi_options = json_decode($r->option_ids);
                    $multi_options = $r->option_ids;
                    if(!empty($multi_options)){
                        //print_r($multi_options);
                        // if($multi_options == null){
                        //     // There was an error
                        //     return error((object) null, 'Bad JSON received', 400);
                        // }  
                        $positive_opt_count=$negative_opt_count=0;                  
                        foreach($multi_options as $mo)
                        {
                            if($mo != 0)
                            {
                                $multi_opt_response_tone = QuestionOptions::select('response_tone')->where('question_id',$r->question_id)->where('id',$mo)->get()->first();
                                if($multi_opt_response_tone->response_tone==1){
                                    $response_tone=1;
                                    $positive_opt_count++;
                                }else{
                                    $response_tone=3;
                                    $negative_opt_count++;
                                }
                            }else{
                                $response_tone=0;
                            }
                        }
                        if($positive_opt_count > $negative_opt_count)
                        {
                            $response_tone=1;
                            $positive_tone_count += 1;
                        }else if($positive_opt_count == $negative_opt_count){
                            $response_tone=2;
                            $neutral_tone_count += 1;
                        }else if($positive_opt_count < $negative_opt_count){
                            $response_tone=3;
                            $negative_tone_count += 1;
                        }
                    }
                }else if($r->question_type==9){
                    ////// Multi questions //////
                    $sub_questions = isset($r->sub_questions) ? $r->sub_questions : [];
                    if(!empty($sub_questions))
                    {
                        $positive_opt_count=$negative_opt_count=0;
                        foreach($sub_questions as $sq)
                        {
                            if(isset($sq->option_id))
                            {
                                if($sq->option_id != 0)
                                {
                                    $get_response_tone = QuestionOptions::select('response_tone')->where('question_id',$r->question_id)->where('id',$sq->option_id)->get()->first();
                                    if($get_response_tone->response_tone==1){
                                        $response_tone=1;
                                        $positive_opt_count++;
                                    }else{
                                        $response_tone=3;
                                        $negative_opt_count++;
                                    }
                                }else{
                                    $response_tone=0;
                                }
                            }else{
                                $response_tone=0;
                            }
                        }
                        if($positive_opt_count > $negative_opt_count)
                        {
                            $response_tone=1;
                            $positive_tone_count += 1;
                        }else if($positive_opt_count == $negative_opt_count){
                            $response_tone=2;
                            $neutral_tone_count += 1;
                        }else if($positive_opt_count < $negative_opt_count){
                            $response_tone=3;
                            $negative_tone_count += 1;
                        }
                    }
                }else if($r->question_type==7){
                    $response_tone=0;
                }
                $json_data['question_id']=$r->question_id;
                $json_data['question_type']=$r->question_type;
                $json_data['rating']=$r->rating;
                $json_data['option_ids']=$r->option_ids;
                $json_data['options_value']=$r->options_value;
                $json_data['feedback_question_ids']=$r->feedback_question_ids;
                $json_data['feedback_question_values']=$r->feedback_question_values;
                $json_data['sub_questions']= isset($r->sub_questions) ? $r->sub_questions : [];
                $json_data['response_tone']=$response_tone;


                // //// Add rating question details ///////
                // if(!empty($r->rating) && is_numeric($r->rating))
                // {
                //     $question_details_data['question_id'] = $r->question_id;
                //     $question_details_data['question_type'] = $r->question_type;
                //     $question_details_data['servey_id'] = $request->servey_id;
                //     $question_details_data['rating'] = $r->rating;
                //     $question_details_data['response_tone'] = $response_tone;
                //     $question_details_data['date'] = date('Y-m-d');
                //     $question_details_data['time'] = date('H:i:s');
                //     $question_details_data['created_at'] = date('Y-m-d H:i:s');
                //     $question_details_data['updated_at'] = date('Y-m-d H:i:s');
                //     $que_details_id = RatingQuestionDetails::insert($question_details_data);
                // }
               

                // //// Add rating feedback details ///////
                // if(!empty($r->feedback_question_ids))
                // {
                //     $fds = $r->feedback_question_ids;
                //     $fd_values = $r->feedback_question_values;

                //     $feedback_que_data=[];
                //     foreach($fds as $k=>$f)
                //     {
                //         $feedback_que_data[] = [
                //             'question_id' => $r->question_id,
                //             'question_type' => $r->question_type,
                //             'servey_id' => $request->servey_id,
                //             'feedback_question_id' => $f,
                //             'feedback_question' => $fd_values[$k],
                //             'date' => date('Y-m-d'),
                //             'time' => date('H:i:s'),
                //             'created_at' => date('Y-m-d H:i:s'),
                //             'updated_at' => date('Y-m-d H:i:s')
                //         ];
                //     }
                //     RatingFeedbackQuestionDetails::insert($feedback_que_data);
                // }
                // //// Add rating question options ///////
                // if(!empty($r->option_ids))
                // {
                //     $opts = $r->option_ids;
                //     $opt_values = $r->options_value;

                //     $option_data=[];
                //     foreach($opts as $o=>$v)
                //     {
                //         $option_data[] = [
                //             'question_id' => $r->question_id,
                //             'question_type' => $r->question_type,
                //             'servey_id' => $request->servey_id,
                //             'option_id' => $v,
                //             'option_value' => $opt_values[$o],
                //             'date' => date('Y-m-d'),
                //             'time' => date('H:i:s'),
                //             'created_at' => date('Y-m-d H:i:s'),
                //             'updated_at' => date('Y-m-d H:i:s')
                //         ];
                //     }
                //     RatingQuestionOptions::insert($option_data);
                // }

                $final[] = $json_data;
            }
            $raings['servey_response']=json_encode($final);

            ///// find performance ///////

            $raings['positive'] = $positive_tone_count;
            $raings['negative'] = $negative_tone_count;
            $raings['nuetral'] = $neutral_tone_count;
            $question_count = $positive_tone_count+$negative_tone_count+$neutral_tone_count;
            $raings['question_count'] = $question_count;
            $positive_tone_performance =  ($question_count!=0) ? $positive_tone_count/$question_count : 0;
            $negative_tone_performance =  ($question_count!=0) ? $negative_tone_count/$question_count : 0;
            $neutral_tone_performance =  ($question_count!=0) ? $neutral_tone_count/$question_count : 0;
            $final_performance = max($positive_tone_performance,$negative_tone_performance,$neutral_tone_performance);
            
            $array = array($positive_tone_count,$negative_tone_count,$neutral_tone_count);
            $maxs = array_keys($array, max($array));
            
            if($maxs[0]==0) 
                $performance =1;
            else if($maxs[0]==1)
                $performance =3;
            else
                $performance =2;

            
            $raings['positive_avg'] = $positive_tone_performance;
            $raings['negative_avg'] = $negative_tone_performance;
            $raings['neutral_avg'] = $neutral_tone_performance;
            $raings['positive_per'] = ($positive_tone_performance*100);
            $raings['negative_per'] = ($negative_tone_performance*100);
            $raings['neutral_per'] = ($neutral_tone_performance*100);


            $raings['performance']=$performance;
            $rating_id = $raings->save();
            if($rating_id)
            {
                if(!empty($request->name) || !empty($request->email) || !empty($request->phone_no))
                {
                    $customer_details = RatingCustomerDetails::create([
                        'rating_id' => $raings->id,
                        'name' => isset($request->name) ? $request->name : '',
                        'email' => isset($request->email) ? $request->email : '',
                        'phone_no' => isset($request->phone_no) ? $request->phone_no : '',
                    ]);
                }
            }

            /*   NOTE   */
            // save houly feedback code is in observer //
            // $date = date('Y-m-d');
            // $time = date('H:i:s');
            // $hour = ltrim(date('H'),'0');
            // $hour_cl = $hour.'_hr';
            // //==== check rating exist or not //////////
            // $today_rating = \App\Models\HourlyRatings::select('id','rating',"$hour_cl")
            // ->where('monitor_id',$request->monitor_id)
            // ->whereDate('date',$date)
            // //->where(DB::raw("(DATE_FORMAT(time,'%H'))"),$hour)
            // ->get()->toArray();
            
            
            // $hourly_rating_id='';
            // for($i=1;$i<=3;$i++)
            // {
            //     if(!empty($today_rating))
            //     {
            //         $hourly_rating_id = array_search($i, array_column($today_rating, 'rating'));
            //     }
                
            //     if($i==1) $new_rating = $positive_tone_count;
            //     else if($i==2) $new_rating = $neutral_tone_count;
            //     else $new_rating = $negative_tone_count;

            //     $p_rating = new \App\Models\HourlyRatings;
            //     $p_rating['company_id']=$request->company_id;
            //     $p_rating['site_id']=$request->site_id;
            //     $p_rating['monitor_id']=$request->monitor_id;
            //     $p_rating['rating'] = $i;
            //     $p_rating['time'] = $time;
                
            //     $p_rating = [
            //         'monitor_id'=>$request->monitor_id,
            //         'company_id' => $request->company_id,
            //         'site_id' => $request->site_id,
            //         'rating' => $i,
            //         'date' => $date,
            //         'time' => $time
            //     ]; 

            //     if($hourly_rating_id != '')
            //     {
            //         //// update rating /////
            //         $current_rating = $today_rating[$hourly_rating_id][$hour_cl];
            //         $p_rating["$hour_cl"] =  $current_rating + $new_rating;
            //     }else{
            //         $p_rating["$hour_cl"] = $new_rating;
            //     }
            //     //print_r($p_rating);
            //     \App\Models\HourlyRatings::updateOrCreate(
            //         ['monitor_id'=>$request->monitor_id,'date' => $date,'rating'=>$i],
            //         $p_rating
            //     );
            // }
               
            if(!empty($rating_id))
            {
                return success($rating_id,'Success',200);
            }
        }
        return error((object)null,'Json not valid',400);
    }

    //// Reset Settings : notification //////
    public function ResetSettingsNotification(Request $request){
 
        $request_parameters = array("monitor_id");
        $token = json_decode(bearerToken($request,$request_parameters));
        if($token->is_error==1)
        {
            return error((object)null,'Missing Parameter '.$token->data,400);
        }

        $tokens = MonitorDeviceTokens::where('monitor_id', $request->monitor_id)->orderBy('id','ASC')->get()->toArray();
      
        $token=array();
        if(!empty($tokens))
        {
            foreach($tokens as $t)
            {
                $token[]=$t['token'];
            }
        }
        if(!empty($token))
        {
            $payload = array(
                "registration_ids" => $token,
                "notification"=>array(
                    "title" => 'Feedback247',
                    "body"=> "Reset settings",
                    "icon" => "ic_launcher",
                    "click_action"=>""
                ),
                "data"=>array(
                    "monitor_id" => $request->monitor_id
                )
            );
            $res = android_push_notification($payload, $token);

            if($res == 1){
               // success code
               return success([],'Success',200);
            }else{
                // fail code
                return success([],'Fail',200);
            } 
        }
    }  
}
?>