<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\Site;
use Config;
use DB;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class DynamicDatabaseConnection
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return \Illuminate\Http\Response|\Illuminate\Http\RedirectResponse
    */

    public function handle($request, Closure $next,$table_name='')
    {
        if(empty(Auth::user()))
        {
            return redirect()->route('login');
        }
        if(Auth::user()->role == 2 || Auth::user()->role == 3)
        {
            Config::set('database.connections.mysql2.database', Auth::user()->db_name);
            
            if(!empty($table_name)){
              if (!Schema::connection('mysql2')->hasTable($table_name)) {
                if($table_name=='sites')
                {
                    Schema::connection('mysql2')->create($table_name, function (Blueprint $table) {
                        $table->increments('id');
                        $table->bigInteger('company_id')->nullable();
                        $table->string('name')->nullable();
                        $table->string('location')->nullable();
                        $table->string('slug')->nullable();
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                    if (!Schema::connection('mysql2')->hasTable('site_users')) {
                        Schema::connection('mysql2')->create('site_users', function (Blueprint $table) {
                            $table->increments('id');
                            $table->bigInteger('site_id')->nullable();
                            $table->string('user_id')->nullable();
                            $table->integer('active')->default(1);
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                }else if($table_name=='servey'){
                    Schema::connection('mysql2')->create($table_name, function (Blueprint $table) {
                        $table->increments('id');
                        $table->string('name')->nullable();
                        $table->integer('type')->nullable()->comment('1 : portrait, 2 : landscape');
                        $table->text('size')->nullable();
                        $table->text('message')->nullable()->comment('Display thankyou msg in app after giving rating.');
                        $table->string('slug')->nullable();
                        $table->integer('ask_customer_details')->default(0);
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                    // Schema::connection('mysql2')->create('questions', function (Blueprint $table) {
                    //     $table->increments('id');
                    //     $table->integer('servey_id')->nullable();
                    //     $table->string('question')->nullable();
                    //     $table->string('description')->nullable();
                    //     $table->string('positive_title')->nullable();
                    //     $table->string('negative_title')->nullable();
                    //     $table->string('icon_id')->nullable();
                    //     $table->string('question_type')->nullable()->comment('1=smiley,2=rating bar,3=multiple image selections,4=scale,5=multiple options ');
                    //     $table->integer('icon_type')->nullable()->comment('for smiley 1 = 1-2-3,for rating bar=1-2-3-4');
                    //     $table->integer('is_multi_seleted')->default(0);
                    //     $table->integer('active')->default(1);
                    //     $table->integer('created_by')->nullable();
                    //     $table->integer('updated_by')->nullable();
                    //     $table->integer('deleted_by')->nullable();
                    //     $table->softDeletes();
                    //     $table->timestamps();
                    // });
                    Schema::connection('mysql2')->create('ratings', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('company_id')->nullable();
                        $table->integer('site_id')->nullable();
                        $table->integer('monitor_id')->nullable();
                        $table->integer('servey_id')->nullable();
                        $table->json('servey_response')->nullable();
                        $table->integer('positive')->default(0);
                        $table->integer('negative')->default(0);
                        $table->integer('nuetral')->default(0);
                        $table->decimal('positive_avg',5,2)->nullable();
                        $table->decimal('negative_avg',5,2)->nullable();
                        $table->decimal('neutral_avg',5,2)->nullable();
                        $table->decimal('positive_per',5,2)->nullable();
                        $table->decimal('negative_per',5,2)->nullable();
                        $table->decimal('neutral_per',5,2)->nullable();
                        $table->integer('question_count')->default(0);
                        $table->integer('performance')->nullable()->comment('1:Good,2:Fair,3:Poor');
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                    Schema::connection('mysql2')->create('hourly_ratings', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('company_id')->nullable();
                        $table->integer('site_id')->nullable();
                        $table->integer('monitor_id')->nullable();
                        $table->integer('rating')->nullable();
                        $table->date('date')->nullable();
                        $table->time('time')->nullable();
                        $table->integer('0_hr')->default(0);
                        $table->integer('1_hr')->default(0);
                        $table->integer('2_hr')->default(0);
                        $table->integer('3_hr')->default(0);
                        $table->integer('4_hr')->default(0);
                        $table->integer('5_hr')->default(0);
                        $table->integer('6_hr')->default(0);
                        $table->integer('7_hr')->default(0);
                        $table->integer('8_hr')->default(0);
                        $table->integer('9_hr')->default(0);
                        $table->integer('10_hr')->default(0);
                        $table->integer('11_hr')->default(0);
                        $table->integer('12_hr')->default(0);
                        $table->integer('13_hr')->default(0);
                        $table->integer('14_hr')->default(0);
                        $table->integer('15_hr')->default(0);
                        $table->integer('16_hr')->default(0);
                        $table->integer('17_hr')->default(0);
                        $table->integer('18_hr')->default(0);
                        $table->integer('19_hr')->default(0);
                        $table->integer('20_hr')->default(0);
                        $table->integer('21_hr')->default(0);
                        $table->integer('22_hr')->default(0);
                        $table->integer('23_hr')->default(0);
                        $table->timestamps();
                    });
                    Schema::connection('mysql2')->create('question_options', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('question_id')->nullable();
                        $table->integer('option_id')->nullable();
                        $table->string('options')->nullable();
                        $table->string('response_tone')->nullable();
                        $table->integer('active')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                    Schema::connection('mysql2')->create('rating_customer_details', function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('rating_id')->nullable();
                        $table->string('name')->nullable();
                        $table->string('email')->nullable();
                        $table->string('phone_no')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }else if($table_name=='feedback_questions'){
                    Schema::connection('mysql2')->create($table_name, function (Blueprint $table) {
                        $table->increments('id');
                        $table->text('question')->nullable();
                        $table->string('question_type')->nullable()->comment('1:positive,2:negative');;
                        $table->string('issue_type')->nullable()->comment('1:maintenance,2:cleanliness,3:service');;
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->integer('active')->default(1);
                        $table->softDeletes();
                        $table->timestamps();
                    });
                }else if($table_name=='questions'){
                    Schema::connection('mysql2')->create($table_name, function (Blueprint $table) {
                        $table->increments('id');
                        $table->integer('servey_id')->nullable();
                        $table->string('question')->nullable();
                        $table->string('description')->nullable();
                        $table->string('positive_title')->nullable();
                        $table->string('negative_title')->nullable();
                        $table->string('icon_id')->nullable();
                        $table->string('question_type')->nullable()->comment('1=smiley,2=rating bar,3=multiple image selections,4=scale,5=multiple options ');
                        $table->integer('icon_type')->nullable()->comment('for smiley 1 = 1-2-3,for rating bar=1-2-3-4');
                        $table->integer('is_multi_seleted')->default(0);
                        $table->integer('other')->default(0);
                        $table->integer('active')->default(1);
                        $table->integer('order')->default(1);
                        $table->integer('created_by')->nullable();
                        $table->integer('updated_by')->nullable();
                        $table->integer('deleted_by')->nullable();
                        $table->softDeletes();
                        $table->timestamps();
                    });
                    if (!Schema::connection('mysql2')->hasTable('question_options')) {
                        Schema::connection('mysql2')->create('question_options', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('question_id')->nullable();
                            $table->integer('option_id')->nullable();
                            $table->string('options')->nullable();
                            $table->integer('response_tone')->default(1);
                            $table->integer('active')->default(1);
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                    if (!Schema::connection('mysql2')->hasTable('servey_feedback_questions')) {
                        Schema::connection('mysql2')->create('servey_feedback_questions', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('servey_id')->nullable();
                            $table->integer('question_id')->nullable();
                            $table->integer('feedback_question_id')->nullable();
                            $table->integer('response_tone')->nullable()->comment('1=positive,2=negative');
                            $table->integer('active')->default(1);
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                }
                else if($table_name=='monitors'){
                    if (!Schema::connection('mysql2')->hasTable('monitor_advertise')) {
                        Schema::connection('mysql2')->create('monitor_advertise', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('monitor_id')->nullable();
                            $table->text('image')->nullable();
                            $table->integer('sort')->nullable();
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                    if (!Schema::connection('mysql2')->hasTable('monitor_config')) {
                        Schema::connection('mysql2')->create('monitor_config', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('monitor_id')->nullable();
                            $table->integer('config_id')->nullable();
                            $table->enum('date_format',['1', '2'])->nullable()->comment('1 : dd/mm/yy, 2 : mm/dd/yy');
                            $table->enum('clock_format',['1', '2'])->nullable()->comment('1 : 12hr, 2 : 24hr');
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                }else{
                    if (!Schema::connection('mysql2')->hasTable('servey_feedback_questions')) {
                        Schema::connection('mysql2')->create('servey_feedback_questions', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('servey_id')->nullable();
                            $table->integer('question_id')->nullable();
                            $table->integer('feedback_question_id')->nullable();
                            $table->integer('response_tone')->nullable()->comment('1=positive,2=negative');
                            $table->integer('active')->default(1);
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                    if (!Schema::connection('mysql2')->hasTable('servey_config')) {
                        Schema::connection('mysql2')->create('servey_config', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('servey_id')->nullable();
                            $table->integer('config_id')->nullable();
                            $table->enum('date_format',['1', '2'])->nullable()->comment('1 : dd/mm/yy, 2 : mm/dd/yy');
                            $table->enum('clock_format',['1', '2'])->nullable()->comment('1 : 12hr, 2 : 24hr');
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                    if (!Schema::connection('mysql2')->hasTable('servey_advertise')) {
                        Schema::connection('mysql2')->create('servey_advertise', function (Blueprint $table) {
                            $table->increments('id');
                            $table->integer('servey_id')->nullable();
                            $table->text('image')->nullable();
                            $table->integer('sort')->nullable();
                            $table->integer('created_by')->nullable();
                            $table->integer('updated_by')->nullable();
                            $table->integer('deleted_by')->nullable();
                            $table->softDeletes();
                            $table->timestamps();
                        });
                    }
                }
                 
                }  
            }
            
        }
        return $next($request);
    }
}
// DB::connection('mysql2');
// Artisan::call('migrate', ['--database' => Auth::user()->db_name]);
// if (!Schema::hasTable('tableName2')) {
//     Site::Schema::create('tableName2', function (Blueprint $table) {
//         $table->increments('id');
//         $table->timestamps();
//     });
// }

//$site = new Site;
//return Site::all();
//return $site->creteTable('Table2');

//Config::set('database.connections.mysql2.database', Auth::user()->db_name);//new database name, you want to connect to.
//return DB::connection('mysql2')->table('test')->get();
//return $databaseName = \DB::connection()->getDatabaseName();
// $defaultConfig = Config('database.connections.mysql2');
// $defaultConfig['database'] = Auth::user()->db_name;

//Config::set('database.connections.mysql2.database', Auth::user()->db_name);
// DB::purge('mysql2');
//DB::reconnect('mysql2');
//$databaseName = Config::get('database.connections');
//return $databaseName;
//return DB::table('test')->get();

// Config::set("database.connections.mysql.host", Auth::user()->db_server);
// Config::set("database.connections.mysql.port", Auth::user()->db_port);
// Config::set("database.connections.mysql.database", Auth::user()->db_name);
// DB::purge('mysql');
// DB::reconnect('mysql');



//Config(['database.connection.second' => $defaultConfig]);

// Config::set("database.connections.mysql.database", Auth::user()->db_name);

// echo $databaseName = \DB::connection()->getDatabaseName();
// //DB::insert("Insert into test('name','address') values('Shivani','Adres 1')");

// $companies = Site::get()->toArray();
// print_r($companies);
// $sites = DB::table('test')
// ->selectRaw('name')
// ->get();
// dd($sites);
// $databaseName = \DB::connection()->getDatabaseName();
// $databaseName = Config::get('database.connections');


// dd($databaseName['mysql']['database']);

//dd($databaseName);



// echo Auth::user()->db_name;
// Config::set('database.connections.club.database', Auth::user()->db_name);

// DB::purge('club');
// DB::reconnect('club');


// Config::set("database.connections.mysql.host", Auth::user()->db_server);
// Config::set("database.connections.mysql.port", Auth::user()->db_port);
// Config::set("database.connections.mysql.database", Auth::user()->db_name);
// DB::purge('mysql');
// DB::reconnect('mysql');// DB::connection('mysql2');
// Artisan::call('migrate', ['--database' => Auth::user()->db_name]);
// if (!Schema::hasTable('tableName2')) {
//     Site::Schema::create('tableName2', function (Blueprint $table) {
//         $table->increments('id');
//         $table->timestamps();
//     });
// }

//$site = new Site;
//return Site::all();
//return $site->creteTable('Table2');

//Config::set('database.connections.mysql2.database', Auth::user()->db_name);//new database name, you want to connect to.
//return DB::connection('mysql2')->table('test')->get();
//return $databaseName = \DB::connection()->getDatabaseName();
// $defaultConfig = Config('database.connections.mysql2');
// $defaultConfig['database'] = Auth::user()->db_name;

//Config::set('database.connections.mysql2.database', Auth::user()->db_name);
// DB::purge('mysql2');
//DB::reconnect('mysql2');
//$databaseName = Config::get('database.connections');
//return $databaseName;
//return DB::table('test')->get();

// Config::set("database.connections.mysql.host", Auth::user()->db_server);
// Config::set("database.connections.mysql.port", Auth::user()->db_port);
// Config::set("database.connections.mysql.database", Auth::user()->db_name);
// DB::purge('mysql');
// DB::reconnect('mysql');



//Config(['database.connection.second' => $defaultConfig]);

// Config::set("database.connections.mysql.database", Auth::user()->db_name);

// echo $databaseName = \DB::connection()->getDatabaseName();
// //DB::insert("Insert into test('name','address') values('Shivani','Adres 1')");

// $companies = Site::get()->toArray();
// print_r($companies);
// $sites = DB::table('test')
// ->selectRaw('name')
// ->get();
// dd($sites);
// $databaseName = \DB::connection()->getDatabaseName();
// $databaseName = Config::get('database.connections');


// dd($databaseName['mysql']['database']);

//dd($databaseName);



// echo Auth::user()->db_name;
// Config::set('database.connections.club.database', Auth::user()->db_name);

// DB::purge('club');
// DB::reconnect('club');


// Config::set("database.connections.mysql.host", Auth::user()->db_server);
// Config::set("database.connections.mysql.port", Auth::user()->db_port);
// Config::set("database.connections.mysql.database", Auth::user()->db_name);
// DB::purge('mysql');
// DB::reconnect('mysql');

